@@ -28,7 +28,7 @@
 class KCMKabCustomFields : public KPIM::KCMDesignerFields
 {
   public:
-    KCMKabCustomFields( QWidget *parent = 0, const char *name = 0 );
+    KCMKabCustomFields( QWidget *parent, const QVariantList &args );
 
   protected:
     QString localUiDir();
