@@ -21,38 +21,37 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <qstring.h>
-#include <qlayout.h>
-#include <qlabel.h>
-#include <qcheckbox.h>
-#include <qvbox.h>
-#include <qgroupbox.h>
-#include <qspinbox.h>
-#include <qtabwidget.h>
-#include <qwhatsthis.h>
-
-#include <kdebug.h>
-#include <kglobal.h>
-#include <kglobalsettings.h>
-#include <klocale.h>
-#include <kiconloader.h>
-#include <kconfig.h>
-#include <kfontdialog.h>
-#include <kpushbutton.h>
-
-#include "colorlistbox.h"
-
 #include "configurecardviewdialog.h"
 
+#include <QtCore/QString>
+#include <QtGui/QCheckBox>
+#include <QtGui/QFrame>
+#include <QtGui/QGridLayout>
+#include <QtGui/QLabel>
+#include <QtGui/QSpinBox>
+#include <Qt3Support/Q3GroupBox>
+
+#include <kcolorscheme.h>
+#include <kconfig.h>
+#include <kdebug.h>
+#include <kfontdialog.h>
+#include <kglobal.h>
+#include <kiconloader.h>
+#include <klocale.h>
+#include <kpushbutton.h>
+#include <ktabwidget.h>
+#include <kvbox.h>
+
+#include <libkdepim/colorlistbox.h>
+
 /////////////////////////////////
 // ConfigureCardViewDialog
 
-ConfigureCardViewWidget::ConfigureCardViewWidget( KABC::AddressBook *ab, QWidget *parent,
-                                                  const char *name )
-  : ViewConfigureWidget( ab, parent, name )
+ConfigureCardViewWidget::ConfigureCardViewWidget( KABC::AddressBook *ab, QWidget *parent )
+  : ViewConfigureWidget( ab, parent )
 {
-  QWidget *page = addPage( i18n( "Look & Feel" ), QString::null,
-                           DesktopIcon( "looknfeel" ) );
+  QWidget *page = addPage( i18n( "Look & Feel" ), QString(),
+                           DesktopIcon( "preferences-desktop-theme" ) );
   mAdvancedPage = new CardViewLookNFeelPage( page );
 }
 
@@ -60,14 +59,14 @@ ConfigureCardViewWidget::~ConfigureCardViewWidget()
 {
 }
 
-void ConfigureCardViewWidget::restoreSettings( KConfig *config )
+void ConfigureCardViewWidget::restoreSettings( const KConfigGroup &config )
 {
   ViewConfigureWidget::restoreSettings( config );
 
   mAdvancedPage->restoreSettings( config );
 }
 
-void ConfigureCardViewWidget::saveSettings( KConfig *config )
+void ConfigureCardViewWidget::saveSettings( KConfigGroup &config )
 {
   ViewConfigureWidget::saveSettings( config );
 
@@ -76,8 +75,8 @@ void ConfigureCardViewWidget::saveSettings( KConfig *config )
 
 ////////////////////////
 // CardViewLookNFeelPage
-CardViewLookNFeelPage::CardViewLookNFeelPage( QWidget *parent, const char *name )
-  : QVBox( parent, name )
+CardViewLookNFeelPage::CardViewLookNFeelPage( QWidget *parent )
+  : KVBox( parent )
 {
   initGUI();
 }
@@ -86,82 +85,81 @@ CardViewLookNFeelPage::~CardViewLookNFeelPage()
 {
 }
 
-void CardViewLookNFeelPage::restoreSettings( KConfig *config )
+void CardViewLookNFeelPage::restoreSettings( const KConfigGroup &config )
 {
   // colors
-  cbEnableCustomColors->setChecked( config->readBoolEntry( "EnableCustomColors", false ) );
-  QColor c;
-  c = KGlobalSettings::baseColor();
-  lbColors->insertItem( new ColorListItem( i18n("Background Color"),
-        config->readColorEntry( "BackgroundColor", &c ) ) );
-  c = colorGroup().foreground();
-  lbColors->insertItem( new ColorListItem( i18n("Text Color"),
-        config->readColorEntry( "TextColor", &c ) ) );
-  c = colorGroup().button();
-  lbColors->insertItem( new ColorListItem( i18n("Header, Border & Separator Color"),
-        config->readColorEntry( "HeaderColor", &c ) ) );
-  c = colorGroup().buttonText();
-  lbColors->insertItem( new ColorListItem( i18n("Header Text Color"),
-        config->readColorEntry( "HeaderTextColor", &c ) ) );
-  c = colorGroup().highlight();
-  lbColors->insertItem( new ColorListItem( i18n("Highlight Color"),
-        config->readColorEntry( "HighlightColor", &c ) ) );
-  c = colorGroup().highlightedText();
-  lbColors->insertItem( new ColorListItem( i18n("Highlighted Text Color"),
-        config->readColorEntry( "HighlightedTextColor", &c ) ) );
+  cbEnableCustomColors->setChecked( config.readEntry( "EnableCustomColors", false ) );
+  QColor c = KColorScheme(QPalette::Active).background().color();
+  lbColors->addColor( i18n("Background Color"), 
+                      config.readEntry( "BackgroundColor", c ) );
+  c = palette().color( QPalette::Foreground );
+  lbColors->addColor( i18n("Text Color"),
+                      config.readEntry( "TextColor", c ) );
+  c = palette().color( QPalette::Button );
+  lbColors->addColor( i18n("Header, Border & Separator Color"),
+                      config.readEntry( "HeaderColor", c ) );
+  c = palette().color( QPalette::ButtonText );
+  lbColors->addColor( i18n("Header Text Color"),
+                      config.readEntry( "HeaderTextColor", c ) );
+  c = palette().color( QPalette::Highlight );
+  lbColors->addColor( i18n("Highlight Color"),
+                      config.readEntry( "HighlightColor", c ) );
+  c = palette().color( QPalette::HighlightedText );
+  lbColors->addColor( i18n("Highlighted Text Color"),
+                      config.readEntry( "HighlightedTextColor", c ) );
 
   enableColors();
 
   // fonts
   QFont fnt = font();
-  updateFontLabel( config->readFontEntry( "TextFont", &fnt ), (QLabel*)lTextFont );
+  updateFontLabel( config.readEntry( "TextFont", fnt ), (QLabel*)lTextFont );
   fnt.setBold( true );
-  updateFontLabel( config->readFontEntry( "HeaderFont",  &fnt ), (QLabel*)lHeaderFont );
-  cbEnableCustomFonts->setChecked( config->readBoolEntry( "EnableCustomFonts", false ) );
+  updateFontLabel( config.readEntry( "HeaderFont",  fnt ), (QLabel*)lHeaderFont );
+  cbEnableCustomFonts->setChecked( config.readEntry( "EnableCustomFonts", false ) );
   enableFonts();
 
   // layout
-  sbMargin->setValue( config->readNumEntry( "ItemMargin", 0 ) );
-  sbSpacing->setValue( config->readNumEntry( "ItemSpacing", 10 ) );
-  sbSepWidth->setValue( config->readNumEntry( "SeparatorWidth", 2 ) );
-  cbDrawSeps->setChecked( config->readBoolEntry( "DrawSeparators", true ) );
-  cbDrawBorders->setChecked( config->readBoolEntry( "DrawBorder", true ) );
+  sbMargin->setValue( config.readEntry( "ItemMargin", 0 ) );
+  sbSpacing->setValue( config.readEntry( "ItemSpacing", 10 ) );
+  sbSepWidth->setValue( config.readEntry( "SeparatorWidth", 2 ) );
+  cbDrawSeps->setChecked( config.readEntry( "DrawSeparators", true ) );
+  cbDrawBorders->setChecked( config.readEntry( "DrawBorder", true ) );
 
   // behaviour
-  cbShowFieldLabels->setChecked( config->readBoolEntry( "DrawFieldLabels", false ) );
-  cbShowEmptyFields->setChecked( config->readBoolEntry( "ShowEmptyFields", false ) );
+  cbShowFieldLabels->setChecked( config.readEntry( "DrawFieldLabels", false ) );
+  cbShowEmptyFields->setChecked( config.readEntry( "ShowEmptyFields", false ) );
 }
 
-void CardViewLookNFeelPage::saveSettings( KConfig *config )
+void CardViewLookNFeelPage::saveSettings( KConfigGroup &config )
 {
   // colors
-  config->writeEntry( "EnableCustomColors", cbEnableCustomColors->isChecked() );
+  config.writeEntry( "EnableCustomColors", cbEnableCustomColors->isChecked() );
   if ( cbEnableCustomColors->isChecked() ) // ?? - Hmmm.
   {
-    config->writeEntry( "BackgroundColor", lbColors->color( 0 ) );
-    config->writeEntry( "TextColor", lbColors->color( 1 ) );
-    config->writeEntry( "HeaderColor", lbColors->color( 2 ) );
-    config->writeEntry( "HeaderTextColor", lbColors->color( 3 ) );
-    config->writeEntry( "HighlightColor", lbColors->color( 4 ) );
-    config->writeEntry( "HighlightedTextColor", lbColors->color( 5 ) );
+    config.writeEntry( "BackgroundColor", lbColors->color( 0 ) );
+    config.writeEntry( "TextColor", lbColors->color( 1 ) );
+    config.writeEntry( "HeaderColor", lbColors->color( 2 ) );
+    config.writeEntry( "HeaderTextColor", lbColors->color( 3 ) );
+    config.writeEntry( "HighlightColor", lbColors->color( 4 ) );
+    config.writeEntry( "HighlightedTextColor", lbColors->color( 5 ) );
   }
   // fonts
-  config->writeEntry( "EnableCustomFonts", cbEnableCustomFonts->isChecked() );
+  config.writeEntry( "EnableCustomFonts", cbEnableCustomFonts->isChecked() );
   if ( cbEnableCustomFonts->isChecked() )
   {
-    config->writeEntry( "TextFont", lTextFont->font() );
-    config->writeEntry( "HeaderFont", lHeaderFont->font() );
+    config.writeEntry( "TextFont", lTextFont->font() );
+    config.writeEntry( "HeaderFont", lHeaderFont->font() );
   }
   // layout
-  config->writeEntry( "ItemMargin", sbMargin->value() );
-  config->writeEntry( "ItemSpacing", sbSpacing->value() );
-  config->writeEntry( "SeparatorWidth", sbSepWidth->value() );
-  config->writeEntry("DrawBorder", cbDrawBorders->isChecked());
-  config->writeEntry("DrawSeparators", cbDrawSeps->isChecked());
+  config.writeEntry( "ItemMargin", sbMargin->value() );
+  config.writeEntry( "ItemSpacing", sbSpacing->value() );
+  config.writeEntry( "SeparatorWidth", sbSepWidth->value() );
+  config.writeEntry("DrawBorder", cbDrawBorders->isChecked());
+  config.writeEntry("DrawSeparators", cbDrawSeps->isChecked());
 
   // behaviour
-  config->writeEntry("DrawFieldLabels", cbShowFieldLabels->isChecked());
-  config->writeEntry("ShowEmptyFields", cbShowEmptyFields->isChecked());
+  config.writeEntry("DrawFieldLabels", cbShowFieldLabels->isChecked());
+  config.writeEntry("ShowEmptyFields", cbShowEmptyFields->isChecked());
 }
 
 void CardViewLookNFeelPage::setTextFont()
@@ -193,73 +191,79 @@ void CardViewLookNFeelPage::initGUI()
   int spacing = KDialog::spacingHint();
   int margin = KDialog::marginHint();
 
-  QTabWidget *tabs = new QTabWidget( this );
+  KTabWidget *tabs = new KTabWidget( this );
 
   // Layout
-  QVBox *loTab = new QVBox( this, "layouttab" );
+  KVBox *loTab = new KVBox( this );
 
   loTab->setSpacing( spacing );
   loTab->setMargin( margin );
 
-  QGroupBox *gbGeneral = new QGroupBox( 1, Qt::Horizontal, i18n("General"), loTab );
+  Q3GroupBox *gbGeneral = new Q3GroupBox( 1, Qt::Horizontal
+    , i18nc("@title:group General layout settings", "General"), loTab );
 
   cbDrawSeps = new QCheckBox( i18n("Draw &separators"), gbGeneral );
 
-  QHBox *hbSW = new QHBox( gbGeneral );
+  KHBox *hbSW = new KHBox( gbGeneral );
   QLabel *lSW = new QLabel( i18n("Separator &width:"), hbSW );
-  sbSepWidth = new QSpinBox( 1, 50, 1, hbSW );
+  sbSepWidth = new QSpinBox( hbSW );
+  sbSepWidth->setRange( 1, 50 );
   lSW->setBuddy( sbSepWidth);
 
-  QHBox *hbPadding = new QHBox( gbGeneral );
+  KHBox *hbPadding = new KHBox( gbGeneral );
   QLabel *lSpacing = new QLabel( i18n("&Padding:"), hbPadding );
-  sbSpacing = new QSpinBox( 0, 100, 1, hbPadding );
+  sbSpacing = new QSpinBox( hbPadding );
+  sbSpacing->setRange( 0, 100 );
   lSpacing->setBuddy( sbSpacing );
 
-  QGroupBox *gbCards = new QGroupBox( 1, Qt::Horizontal, i18n("Cards"), loTab );
+  Q3GroupBox *gbCards = new Q3GroupBox( 1, Qt::Horizontal, i18n("Cards"), loTab );
 
-  QHBox *hbMargin = new QHBox( gbCards );
+  KHBox *hbMargin = new KHBox( gbCards );
   QLabel *lMargin = new QLabel( i18n("&Margin:"), hbMargin );
-  sbMargin = new QSpinBox( 0, 100, 1, hbMargin );
+  sbMargin = new QSpinBox( hbMargin );
+  sbMargin->setRange( 0, 100 );
   lMargin->setBuddy( sbMargin );
 
   cbDrawBorders = new QCheckBox( i18n("Draw &borders"), gbCards );
 
   loTab->setStretchFactor( new QWidget( loTab ), 1 );
-
-  QWhatsThis::add( sbMargin, i18n(
+  QString text = i18n(
         "The item margin is the distance (in pixels) between the item edge and the item data. Most noticeably, "
         "incrementing the item margin will add space between the focus rectangle and the item data."
-        ) );
-  QWhatsThis::add( lMargin, QWhatsThis::textFor( sbMargin ) );
-  QWhatsThis::add( sbSpacing, i18n(
+        );
+  sbMargin->setWhatsThis( text  );
+  lMargin->setWhatsThis( text );
+  text = i18n(
         "The item spacing decides the distance (in pixels) between the items and anything else: the view "
         "borders, other items or column separators."
-        ) );
-  QWhatsThis::add( lSpacing, QWhatsThis::textFor( sbSpacing ) );
-  QWhatsThis::add( sbSepWidth, i18n("Sets the width of column separators") );
-  QWhatsThis::add( lSW, QWhatsThis::textFor( sbSepWidth ) );
+        );
+  sbSpacing->setWhatsThis( text );
+  lSpacing->setWhatsThis( text );
+  text = i18n("Sets the width of column separators");
+  sbSepWidth->setWhatsThis( text );
+  lSW->setWhatsThis( text );
 
-  tabs->addTab( loTab, i18n("&Layout") );
+  tabs->addTab( loTab, i18n("Layout") );
 
   // Colors
-  QVBox *colorTab = new QVBox( this, "colortab" );
+  KVBox *colorTab = new KVBox( this );
   colorTab->setSpacing( spacing );
   colorTab->setMargin( spacing );
   cbEnableCustomColors = new QCheckBox( i18n("&Enable custom colors"), colorTab );
   connect( cbEnableCustomColors, SIGNAL(clicked()), this, SLOT(enableColors()) );
-  lbColors = new ColorListBox( colorTab );
-  tabs->addTab( colorTab, i18n("&Colors") );
+  lbColors = new KPIM::ColorListBox( colorTab );
+  tabs->addTab( colorTab, i18n("Colors") );
 
-  QWhatsThis::add( cbEnableCustomColors, i18n(
+  cbEnableCustomColors->setWhatsThis( i18n(
         "If custom colors is enabled, you may choose the colors for the view below. "
         "Otherwise colors from your current KDE color scheme are used."
         ) );
-  QWhatsThis::add( lbColors, i18n(
+  lbColors->setWhatsThis( i18n(
         "Double click or press RETURN on a item to select a color for the related strings in the view."
         ) );
 
   // Fonts
-  QVBox *fntTab = new QVBox( this, "fonttab" );
+  KVBox *fntTab = new KVBox( this );
 
   fntTab->setSpacing( spacing );
   fntTab->setMargin( spacing );
@@ -268,36 +272,41 @@ void CardViewLookNFeelPage::initGUI()
   connect( cbEnableCustomFonts, SIGNAL(clicked()), this, SLOT(enableFonts()) );
 
   vbFonts = new QWidget( fntTab );
-  QGridLayout *gFnts = new QGridLayout( vbFonts, 2, 3 );
+  QGridLayout *gFnts = new QGridLayout( vbFonts );
   gFnts->setSpacing( spacing );
-  gFnts->setAutoAdd( true );
-  gFnts->setColStretch( 1, 1 );
+  gFnts->setColumnStretch( 1, 1 );
   QLabel *lTFnt = new QLabel( i18n("&Text font:"), vbFonts );
+  gFnts->addWidget( lTFnt, 0, 0 );
   lTextFont = new QLabel( vbFonts );
   lTextFont->setFrameStyle( QFrame::Panel|QFrame::Sunken );
+  gFnts->addWidget( lTextFont, 0, 1 );
   btnFont = new KPushButton( i18n("Choose..."), vbFonts );
   lTFnt->setBuddy( btnFont );
   connect( btnFont, SIGNAL(clicked()), this, SLOT(setTextFont()) );
+  gFnts->addWidget( btnFont, 0, 2 );
 
   QLabel *lHFnt = new QLabel( i18n("&Header font:"), vbFonts );
+  gFnts->addWidget( lTFnt, 1, 0 );
   lHeaderFont = new QLabel( vbFonts );
   lHeaderFont->setFrameStyle( QFrame::Panel|QFrame::Sunken );
+  gFnts->addWidget( lHeaderFont, 1, 1 );
   btnHeaderFont = new KPushButton( i18n("Choose..."), vbFonts );
   lHFnt->setBuddy( btnHeaderFont );
   connect( btnHeaderFont, SIGNAL(clicked()), this, SLOT(setHeaderFont()) );
+  gFnts->addWidget( btnHeaderFont, 1, 2 );
 
   fntTab->setStretchFactor( new QWidget( fntTab ), 1 );
 
-  QWhatsThis::add( cbEnableCustomFonts, i18n(
+  cbEnableCustomFonts->setWhatsThis( i18n(
         "If custom fonts are enabled, you may choose which fonts to use for this view below. "
         "Otherwise the default KDE font will be used, in bold style for the header and "
         "normal style for the data."
         ) );
 
-  tabs->addTab( fntTab, i18n("&Fonts") );
+  tabs->addTab( fntTab, i18n("Fonts") );
 
   // Behaviour
-  QVBox *behaviourTab = new QVBox( this );
+  KVBox *behaviourTab = new KVBox( this );
   behaviourTab->setMargin( margin );
   behaviourTab->setSpacing( spacing );
 
@@ -306,7 +315,7 @@ void CardViewLookNFeelPage::initGUI()
 
   behaviourTab->setStretchFactor( new QWidget( behaviourTab ), 1 );
 
-  tabs->addTab( behaviourTab, i18n("Be&havior") );
+  tabs->addTab( behaviourTab, i18n("Behavior") );
 
 }
 
