@@ -35,7 +35,7 @@ class GNOKIIXXPort : public KAB::XXPort
 
     QString identifier() const { return "gnokii"; }
 
-  public slots:
+  public Q_SLOTS:
     bool exportContacts( const KABC::AddresseeList &list, const QString &data );
     KABC::Addressee::List importContacts( const QString &data ) const;
 };
