@@ -1,7 +1,7 @@
 /*
  *  editdlg.cpp  -  dialog to create or modify an alarm or alarm template
  *  Program:  kalarm
- *  Copyright © 2001-2008 by David Jarvie <djarvie@kde.org>
+ *  Copyright © 2001-2009 by David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,14 +19,17 @@
  */
 
 #include "kalarm.h"
+#include "editdlg.moc"
+#include "editdlgprivate.moc"
+#include "editdlgtypes.h"
 
 #include "alarmcalendar.h"
 #include "alarmresources.h"
 #include "alarmtimewidget.h"
+#include "autoqpointer.h"
 #include "buttongroup.h"
 #include "checkbox.h"
 #include "deferdlg.h"
-#include "dialogscroll.h"
 #include "functions.h"
 #include "kalarmapp.h"
 #include "latecancel.h"
@@ -39,12 +42,26 @@
 #include "reminder.h"
 #include "shellprocess.h"
 #include "spinbox.h"
+#include "stackedwidgets.h"
 #include "templatepickdlg.h"
 #include "timeedit.h"
 #include "timespinbox.h"
-#include "editdlg.moc"
-#include "editdlgprivate.moc"
-#include "editdlgtypes.h"
+
+#include <libkdepim/maillistdrag.h>
+#include <libkdepim/kvcarddrag.h>
+#include <kcal/period.h>
+#include <kcal/icaldrag.h>
+
+#include <kglobal.h>
+#include <klocale.h>
+#include <kconfig.h>
+#include <kfiledialog.h>
+#include <kpushbutton.h>
+#include <kmessagebox.h>
+#include <khbox.h>
+#include <kvbox.h>
+#include <kwindowsystem.h>
+#include <kdebug.h>
 
 #include <QLabel>
 #include <QDir>
@@ -56,28 +73,17 @@
 #include <QVBoxLayout>
 #include <QDragEnterEvent>
 #include <QResizeEvent>
-#include <QScrollBar>
 #include <QShowEvent>
-
-#include <kglobal.h>
-#include <klocale.h>
-#include <kconfig.h>
-#include <kfiledialog.h>
-#include <kmessagebox.h>
-#include <ktabwidget.h>
-#include <khbox.h>
-#include <kvbox.h>
-#include <kwindowsystem.h>
-#include <kdebug.h>
-
-#include <libkdepim/maillistdrag.h>
-#include <libkdepim/kvcarddrag.h>
-#include <kcal/icaldrag.h>
+#include <QStackedWidget>
+#include <QScrollBar>
+#include <QTimer>
 
 using namespace KCal;
 
 static const char EDIT_DIALOG_NAME[] = "EditDialog";
 static const char TEMPLATE_DIALOG_NAME[] = "EditTemplateDialog";
+static const char EDIT_MORE_GROUP[] = "ShowOpts";
+static const char EDIT_MORE_KEY[]   = "EditMore";
 static const int  maxDelayTime = 99*60 + 59;    // < 100 hours
 
 inline QString recurText(const KAEvent& event)
@@ -103,6 +109,7 @@ EditAlarmDlg* EditAlarmDlg::create(bool Template, Type type, bool newAlarm, QWid
 		case DISPLAY:  return new EditDisplayAlarmDlg(Template, newAlarm, parent, getResource);
 		case COMMAND:  return new EditCommandAlarmDlg(Template, newAlarm, parent, getResource);
 		case EMAIL:    return new EditEmailAlarmDlg(Template, newAlarm, parent, getResource);
+		default:  break;
 	}
 	return 0;
 }
@@ -147,6 +154,8 @@ EditAlarmDlg::EditAlarmDlg(bool Template, KAEvent::Action action, QWidget* paren
 	  mTemplate(Template),
 	  mDesiredReadOnly(false),
 	  mReadOnly(false),
+	  mShowingMore(true),
+	  mWantRepaint(false),
 	  mSavedEvent(0)
 {
 	init(0, getResource);
@@ -169,6 +178,8 @@ EditAlarmDlg::EditAlarmDlg(bool Template, const KAEvent* event, QWidget* parent,
 	  mTemplate(Template),
 	  mDesiredReadOnly(readOnly),
 	  mReadOnly(readOnly),
+	  mShowingMore(true),
+	  mWantRepaint(false),
 	  mSavedEvent(0)
 {
 	init(event, getResource);
@@ -206,11 +217,14 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 	else
 		caption = type_caption(newAlarm);
 	setCaption(caption);
-	setButtons((mReadOnly ? Cancel|Try : mTemplate ? Ok|Cancel|Try : Ok|Cancel|Try|Default));
+	setButtons((mReadOnly ? Cancel|Try|Default : mTemplate ? Ok|Cancel|Try|Default : Ok|Cancel|Try|Help|Default));
 	setDefaultButton(mReadOnly ? Cancel : Ok);
-	setButtonText(Default, i18nc("@action:button", "Load Template..."));
+	setButtonText(Help, i18nc("@action:button", "Load Template..."));
+	setButtonIcon(Help, KIcon());
+	setButtonIcon(Default, KIcon());
 	connect(this, SIGNAL(tryClicked()), SLOT(slotTry()));
 	connect(this, SIGNAL(defaultClicked()), SLOT(slotDefault()));
+	connect(this, SIGNAL(helpClicked()), SLOT(slotHelp()));
 	KVBox* mainWidget = new KVBox(this);
 	mainWidget->setMargin(0);
 	setMainWidget(mainWidget);
@@ -223,13 +237,15 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 		label->setFixedSize(label->sizeHint());
 		mTemplateName = new KLineEdit(box);
 		mTemplateName->setReadOnly(mReadOnly);
+		connect(mTemplateName, SIGNAL(userTextChanged(const QString&)), SLOT(contentsChanged()));
 		label->setBuddy(mTemplateName);
 		box->setWhatsThis(i18nc("@info:whatsthis", "Enter the name of the alarm template"));
 		box->setFixedHeight(box->sizeHint().height());
 	}
 	mTabs = new KTabWidget(mainWidget);
+	mTabScrollGroup = new StackedScrollGroup(this, mTabs);
 
-	DialogScroll<EditAlarmDlg>* mainScroll = new DialogScroll<EditAlarmDlg>();
+	StackedScrollWidget* mainScroll = new StackedScrollWidget(mTabScrollGroup);
 	mTabs->addTab(mainScroll, i18nc("@title:tab", "Alarm"));
 	mMainPageIndex = 0;
 	PageFrame* mainPage = new PageFrame(mainScroll);
@@ -240,7 +256,7 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 	topLayout->setSpacing(spacingHint());
 
 	// Recurrence tab
-	DialogScroll<EditAlarmDlg>* recurScroll = new DialogScroll<EditAlarmDlg>();
+	StackedScrollWidget* recurScroll = new StackedScrollWidget(mTabScrollGroup);
 	mTabs->addTab(recurScroll, QString());
 	mRecurPageIndex = 1;
 	KVBox* recurTab = new KVBox();
@@ -251,6 +267,7 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 	connect(mRecurrenceEdit, SIGNAL(typeChanged(int)), SLOT(slotRecurTypeChange(int)));
 	connect(mRecurrenceEdit, SIGNAL(frequencyChanged()), SLOT(slotRecurFrequencyChange()));
 	connect(mRecurrenceEdit, SIGNAL(repeatNeedsInitialisation()), SLOT(slotSetSubRepetition()));
+	connect(mRecurrenceEdit, SIGNAL(contentsChanged()), SLOT(contentsChanged()));
 
 	// Controls specific to the alarm type
 	QGroupBox* actionBox = new QGroupBox(i18nc("@title:group", "Action"), mainPage);
@@ -297,6 +314,7 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 		grid->setSpacing(spacingHint());
 		mTemplateTimeGroup = new ButtonGroup(templateTimeBox);
 		connect(mTemplateTimeGroup, SIGNAL(buttonSet(QAbstractButton*)), SLOT(slotTemplateTimeType(QAbstractButton*)));
+		connect(mTemplateTimeGroup, SIGNAL(buttonSet(QAbstractButton*)), SLOT(contentsChanged()));
 
 		mTemplateDefaultTime = new RadioButton(i18nc("@option:radio", "Default time"), templateTimeBox);
 		mTemplateDefaultTime->setFixedSize(mTemplateDefaultTime->sizeHint());
@@ -320,6 +338,7 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 		mTemplateTime->setWhatsThis(i18nc("@info:whatsthis",
 		      "<para>Enter the start time for alarms based on this template.</para><para>%1</para>",
 		      TimeSpinBox::shiftWhatsThis()));
+		connect(mTemplateTime, SIGNAL(valueChanged(int)), SLOT(contentsChanged()));
 		box->setStretchFactor(new QWidget(box), 1);    // left adjust the controls
 		box->setFixedHeight(box->sizeHint().height());
 		grid->addWidget(box, 0, 1, Qt::AlignLeft);
@@ -345,6 +364,7 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 		mTemplateTimeAfter->setValue(1439);
 		mTemplateTimeAfter->setFixedSize(mTemplateTimeAfter->sizeHint());
 		mTemplateTimeAfter->setReadOnly(mReadOnly);
+		connect(mTemplateTimeAfter, SIGNAL(valueChanged(int)), SLOT(contentsChanged()));
 		mTemplateTimeAfter->setWhatsThis(i18nc("@info:whatsthis", "<para>%1</para><para>%2</para>",
 		                                       AlarmTimeWidget::i18n_TimeAfterPeriod(), TimeSpinBox::shiftWhatsThis()));
 		box->setFixedHeight(box->sizeHint().height());
@@ -356,51 +376,74 @@ void EditAlarmDlg::init(const KAEvent* event, bool newAlarm)
 	{
 		mTimeWidget = new AlarmTimeWidget(i18nc("@title:group", "Time"), AlarmTimeWidget::AT_TIME, mainPage);
 		connect(mTimeWidget, SIGNAL(dateOnlyToggled(bool)), SLOT(slotAnyTimeToggled(bool)));
+		connect(mTimeWidget, SIGNAL(changed(const KDateTime&)), SLOT(contentsChanged()));
 		topLayout->addWidget(mTimeWidget);
 	}
 
+	// Optional controls depending on More/Less Options button
+	mMoreOptions = new QFrame(mainPage);
+	mMoreOptions->setFrameStyle(QFrame::NoFrame);
+	topLayout->addWidget(mMoreOptions);
+	QVBoxLayout* moreLayout = new QVBoxLayout(mMoreOptions);
+	moreLayout->setMargin(0);
+	moreLayout->setSpacing(spacingHint());
+
 	// Reminder
-	mReminder = createReminder(mainPage);
+	mReminder = createReminder(mMoreOptions);
 	if (mReminder)
 	{
 		mReminder->setFixedSize(mReminder->sizeHint());
-		topLayout->addWidget(mReminder, 0, Qt::AlignLeft);
+		connect(mReminder, SIGNAL(changed()), SLOT(contentsChanged()));
+		moreLayout->addWidget(mReminder, 0, Qt::AlignLeft);
+		if (mTimeWidget)
+			connect(mTimeWidget, SIGNAL(changed(const KDateTime&)), mReminder, SLOT(setDefaultUnits(const KDateTime&)));
 	}
 
 	// Late cancel selector - default = allow late display
-	mLateCancel = new LateCancelSelector(true, mainPage);
-	topLayout->addWidget(mLateCancel, 0, Qt::AlignLeft);
+	mLateCancel = new LateCancelSelector(true, mMoreOptions);
+	connect(mLateCancel, SIGNAL(changed()), SLOT(contentsChanged()));
+	moreLayout->addWidget(mLateCancel, 0, Qt::AlignLeft);
 
 	PackedLayout* playout = new PackedLayout(Qt::AlignJustify);
 	playout->setSpacing(2*spacingHint());
-	topLayout->addLayout(playout);
+	moreLayout->addLayout(playout);
 
 	// Acknowledgement confirmation required - default = no confirmation
-	CheckBox* confirmAck = type_createConfirmAckCheckbox(mainPage);
+	CheckBox* confirmAck = type_createConfirmAckCheckbox(mMoreOptions);
 	if (confirmAck)
 	{
 		confirmAck->setFixedSize(confirmAck->sizeHint());
+		connect(confirmAck, SIGNAL(toggled(bool)), SLOT(contentsChanged()));
 		playout->addWidget(confirmAck);
 	}
 
 	if (theApp()->korganizerEnabled())
 	{
 		// Show in KOrganizer checkbox
-		mShowInKorganizer = new CheckBox(i18n_chk_ShowInKOrganizer(), mainPage);
+		mShowInKorganizer = new CheckBox(i18n_chk_ShowInKOrganizer(), mMoreOptions);
 		mShowInKorganizer->setFixedSize(mShowInKorganizer->sizeHint());
+		connect(mShowInKorganizer, SIGNAL(toggled(bool)), SLOT(contentsChanged()));
 		mShowInKorganizer->setWhatsThis(i18nc("@info:whatsthis", "Check to copy the alarm into KOrganizer's calendar"));
 		playout->addWidget(mShowInKorganizer);
 	}
 
 	setButtonWhatsThis(Ok, i18nc("@info:whatsthis", "Schedule the alarm at the specified time."));
 
+	// Hide optional controls
+	KConfigGroup config(KGlobal::config(), EDIT_MORE_GROUP);
+	showOptions(config.readEntry(EDIT_MORE_KEY, false));
+
 	// Initialise the state of all controls according to the specified event, if any
 	initValues(event);
 	if (mTemplateName)
 		mTemplateName->setFocus();
 
-	// Save the initial state of all controls so that we can later tell if they have changed
-	saveState((event && (mTemplate || !event->isTemplate())) ? event : 0);
+	if (!newAlarm)
+	{
+		// Save the initial state of all controls so that we can later tell if they have changed
+		saveState((event && (mTemplate || !event->isTemplate())) ? event : 0);
+		contentsChanged();    // enable/disable OK button
+	}
 
 	// Note the current desktop so that the dialog can be shown on it.
 	// If a main window is visible, the dialog will by KDE default always appear on its
@@ -499,7 +542,7 @@ void EditAlarmDlg::initValues(const KAEvent* event)
 		if (mShowInKorganizer)
 			mShowInKorganizer->setChecked(event->copyToKOrganizer());
 		type_initValues(event);
-		mRecurrenceEdit->set(*event, (mTemplate || event->isTemplate()));   // must be called after mTimeWidget is set up, to ensure correct date-only enabling
+		mRecurrenceEdit->set(*event);   // must be called after mTimeWidget is set up, to ensure correct date-only enabling
 		mTabs->setTabText(mRecurPageIndex, recurText(*event));
 	}
 	else
@@ -521,12 +564,43 @@ void EditAlarmDlg::initValues(const KAEvent* event)
 		mRecurrenceEdit->setDefaults(defaultTime);   // must be called after mTimeWidget is set up, to ensure correct date-only enabling
 		slotRecurFrequencyChange();      // update the Recurrence text
 	}
+	if (mReminder  &&  mTimeWidget)
+		mReminder->setDefaultUnits(mTimeWidget->getDateTime(0, false, false));
 
 	if (!deferGroupVisible  &&  mDeferGroup)
 		mDeferGroup->hide();
 
 	bool empty = AlarmCalendar::resources()->events(KCalEvent::TEMPLATE).isEmpty();
-	enableButton(Default, !empty);
+	enableButton(Help, !empty);   // Load Templates button
+}
+
+/******************************************************************************
+* Initialise various values in the New Alarm dialogue.
+*/
+void EditAlarmDlg::setTime(const DateTime& start)
+{
+	mTimeWidget->setDateTime(start);
+}
+void EditAlarmDlg::setRecurrence(const KARecurrence& recur, int subRepeatInterval, int subRepeatCount)
+{
+	KAEvent event;
+	event.setTime(mTimeWidget->getDateTime(0, false, false));
+	event.setRecurrence(recur);
+	event.setRepetition(subRepeatInterval, subRepeatCount - 1);
+	mRecurrenceEdit->set(event);
+}
+void EditAlarmDlg::setRepeatAtLogin()
+{
+	mRecurrenceEdit->setRepeatAtLogin();
+}
+void EditAlarmDlg::setLateCancel(int minutes)
+{
+	mLateCancel->setMinutes(minutes, mTimeWidget->getDateTime(0, false, false).isDateOnly(),
+	                        TimePeriod::HoursMinutes);
+}
+void EditAlarmDlg::setShowInKOrganizer(bool show)
+{
+	mShowInKorganizer->setChecked(show);
 }
 
 /******************************************************************************
@@ -573,6 +647,7 @@ void EditAlarmDlg::saveState(const KAEvent* event)
 	if (mShowInKorganizer)
 		mSavedShowInKorganizer = mShowInKorganizer->isChecked();
 	mSavedRecurrenceType   = mRecurrenceEdit->repeatType();
+	mSavedDeferTime        = mDeferDateTime.kDateTime();
 }
 
 /******************************************************************************
@@ -619,6 +694,19 @@ bool EditAlarmDlg::stateChanged() const
 	return false;
 }
 
+/******************************************************************************
+* Called whenever any of the controls changes state.
+* Enable or disable the OK button depending on whether any controls have a
+* different state from their initial state.
+*/
+void EditAlarmDlg::contentsChanged()
+{
+	// Don't do anything if it's a new alarm or we're still initialising
+	// (i.e. mSavedEvent null).
+	if (mSavedEvent  &&  button(Ok))
+		button(Ok)->setEnabled(stateChanged() || mDeferDateTime.kDateTime() != mSavedDeferTime);
+}
+
 /******************************************************************************
 * Get the currently entered dialog data.
 * The data is returned in the supplied KAEvent instance.
@@ -664,7 +752,8 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 			dt = KDateTime(QDate(2000,1,1), mTemplateTime->time());
 	}
 
-	type_setEvent(event, dt, text, (trial ? 0 : mLateCancel->minutes()), trial);
+	int lateCancel = (trial || !mLateCancel->isEnabled()) ? 0 : mLateCancel->minutes();
+	type_setEvent(event, dt, text, lateCancel, trial);
 
 	if (!trial)
 	{
@@ -715,8 +804,8 @@ void EditAlarmDlg::setEvent(KAEvent& event, const QString& text, bool trial)
 */
 int EditAlarmDlg::getAlarmFlags() const
 {
-	return (mShowInKorganizer && mShowInKorganizer->isChecked()                  ? KAEvent::COPY_KORGANIZER : 0)
-	     | (mRecurrenceEdit->repeatType() == RecurrenceEdit::AT_LOGIN            ? KAEvent::REPEAT_AT_LOGIN : 0)
+	return (mShowInKorganizer && mShowInKorganizer->isEnabled() && mShowInKorganizer->isChecked() ? KAEvent::COPY_KORGANIZER : 0)
+	     | (mRecurrenceEdit->repeatType() == RecurrenceEdit::AT_LOGIN                 ? KAEvent::REPEAT_AT_LOGIN : 0)
 	     | ((mTemplate ? mTemplateAnyTime->isChecked() : mAlarmDateTime.isDateOnly()) ? KAEvent::ANY_TIME : 0);
 }
 
@@ -728,7 +817,6 @@ int EditAlarmDlg::getAlarmFlags() const
 void EditAlarmDlg::showEvent(QShowEvent* se)
 {
 	KDialog::showEvent(se);
-	DialogScroll<EditAlarmDlg>::initMinimumHeight(const_cast<EditAlarmDlg*>(this));
 	if (!mDeferGroupHeight)
 	{
 		if (mDeferGroup)
@@ -739,24 +827,41 @@ void EditAlarmDlg::showEvent(QShowEvent* se)
 			bool defer = mDeferGroup && !mDeferGroup->isHidden();
 			s.setHeight(s.height() + (defer ? mDeferGroupHeight : 0));
 			if (!defer)
-				DialogScroll<EditAlarmDlg>::setSized();
+				mTabScrollGroup->setSized();
 			resize(s);
 		}
 	}
+	slotResize();
 	KWindowSystem::setOnDesktop(winId(), mDesktop);    // ensure it displays on the desktop expected by the user
 }
 
 /******************************************************************************
-* Return the minimum size for the dialog.
-* If the minimum size would be too high to fit the desktop, the tab contents
-* are made scrollable.
+* Called when the dialog is closed.
 */
-QSize EditAlarmDlg::minimumSizeHint() const
+void EditAlarmDlg::closeEvent(QCloseEvent* ce)
 {
-	QSize s = DialogScroll<EditAlarmDlg>::initMinimumHeight(const_cast<EditAlarmDlg*>(this));
-	if (s.isValid())
-		return s;
-	return KDialog::minimumSizeHint();
+	emit rejected();
+	KDialog::closeEvent(ce);
+}
+
+/******************************************************************************
+* Update the tab sizes (again) and if the resized dialog height is greater
+* than the minimum, resize it again. This is necessary because (a) resizing
+* tabs doesn't always work properly the first time, and (b) resizing to the
+* minimum size hint doesn't always work either.
+*/
+void EditAlarmDlg::slotResize()
+{
+	QSize s = mTabScrollGroup->adjustSize(true);
+	mWantRepaint = true;
+	s = minimumSizeHint();
+	if (height() > s.height())
+	{
+		// Resize to slightly greater than the minimum height.
+		// This is for some unknown reason necessary, since
+		// sometimes resizing to the minimum height fails.
+		resize(s.width(), s.height() + 2);
+	}
 }
 
 /******************************************************************************
@@ -773,6 +878,9 @@ void EditAlarmDlg::resizeEvent(QResizeEvent* re)
 		KAlarm::writeConfigWindowSize(mTemplate ? TEMPLATE_DIALOG_NAME : EDIT_DIALOG_NAME, s);
 	}
 	KDialog::resizeEvent(re);
+	// Sometimes only the dialog background is shown when first displayed.
+	// Force a repaint after the size has been adjusted by slotResize().
+	QTimer::singleShot(0, this, SLOT(update()));
 }
 
 /******************************************************************************
@@ -930,9 +1038,12 @@ bool EditAlarmDlg::validate()
 		if (!mResourceEventId.isEmpty())
 		{
 			mResource = AlarmCalendar::resources()->resourceForEvent(mResourceEventId);
-			AlarmResource::Type type = mTemplate ? AlarmResource::TEMPLATE : AlarmResource::ACTIVE;
-			if (mResource->alarmType() != type)
-				mResource = 0;   // event may have expired while dialog was open
+			if (mResource)
+			{
+				AlarmResource::Type type = mTemplate ? AlarmResource::TEMPLATE : AlarmResource::ACTIVE;
+				if (mResource->alarmType() != type)
+					mResource = 0;   // event may have expired while dialog was open
+			}
 		}
 		bool cancelled = false;
 		if (!mResource  ||  !mResource->writable())
@@ -943,7 +1054,7 @@ bool EditAlarmDlg::validate()
 		if (!mResource)
 		{
 			if (!cancelled)
-				KMessageBox::sorry(this, i18nc("@info", "You must select a resource to save the alarm in"));
+				KMessageBox::sorry(this, i18nc("@info", "You must select a calendar to save the alarm in"));
 			return false;
 		}
 	}
@@ -982,11 +1093,60 @@ void EditAlarmDlg::slotTrySuccess()
 *  Called when the Load Template button is clicked.
 *  Prompt to select a template and initialise the dialog with its contents.
 */
+void EditAlarmDlg::slotHelp()
+{
+	EventListModel::Type type;
+	switch (mAlarmType)
+	{
+		case KAEvent::FILE:
+		case KAEvent::MESSAGE:  type = EventListModel::DISPLAY;  break;
+		case KAEvent::COMMAND:  type = EventListModel::COMMAND;  break;
+		case KAEvent::EMAIL:    type = EventListModel::EMAIL;  break;
+		default:
+			return;
+	}
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of EditAlarmDlg, and on return from this function).
+	AutoQPointer<TemplatePickDlg> dlg = new TemplatePickDlg(type, this);
+	if (dlg->exec() == QDialog::Accepted)
+		initValues(dlg->selectedTemplate());
+}
+
+/******************************************************************************
+* Called when the More Options or Less Options buttons are clicked.
+* Show/hide the optional options and swap the More/Less buttons, and save the
+* new setting as the default from now on.
+*/
 void EditAlarmDlg::slotDefault()
 {
-	TemplatePickDlg dlg(this);
-	if (dlg.exec() == QDialog::Accepted)
-		initValues(dlg.selectedTemplate());
+	showOptions(!mShowingMore);
+	KConfigGroup config(KGlobal::config(), EDIT_MORE_GROUP);
+	config.writeEntry(EDIT_MORE_KEY, mShowingMore);
+}
+
+/******************************************************************************
+* Show/hide the optional options and swap the More/Less buttons.
+*/
+void EditAlarmDlg::showOptions(bool more)
+{
+	kDebug() << (more ? "More" : "Less");
+	if (more)
+	{
+		mMoreOptions->show();
+		setButtonText(Default, i18nc("@action:button", "Less Options <<"));
+	}
+	else
+	{
+		mMoreOptions->hide();
+		setButtonText(Default, i18nc("@action:button", "More Options >>"));
+	}
+	if (mTimeWidget)
+		mTimeWidget->showMoreOptions(more);
+	type_showOptions(more);
+	mRecurrenceEdit->showMoreOptions(more);
+	mShowingMore = more;
+	QTimer::singleShot(0, this, SLOT(slotResize()));
 }
 
 /******************************************************************************
@@ -1026,8 +1186,12 @@ void EditAlarmDlg::slotEditDeferral()
 	}
 
 	bool deferred = mDeferDateTime.isValid();
-	DeferAlarmDlg deferDlg((deferred ? mDeferDateTime : DateTime(now.addSecs(60))), deferred, this);
-	deferDlg.setObjectName("EditDeferDlg");    // used by LikeBack
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of EditAlarmDlg, and on return from this function).
+	AutoQPointer<DeferAlarmDlg> deferDlg = new DeferAlarmDlg((deferred ? mDeferDateTime : DateTime(now.addSecs(60).toTimeSpec(start.timeSpec()))),
+	                                                         start.isDateOnly(), deferred, this);
+	deferDlg->setObjectName("EditDeferDlg");    // used by LikeBack
 	if (limit)
 	{
 		// Don't allow deferral past the next recurrence
@@ -1038,12 +1202,13 @@ void EditAlarmDlg::slotEditDeferral()
 			if (KDateTime::currentUtcDateTime() < remindTime)
 				start = remindTime;
 		}
-		deferDlg.setLimit(start.addSecs(-60));
+		deferDlg->setLimit(start.addSecs(-60));
 	}
-	if (deferDlg.exec() == QDialog::Accepted)
+	if (deferDlg->exec() == QDialog::Accepted)
 	{
-		mDeferDateTime = deferDlg.getDateTime();
+		mDeferDateTime = deferDlg->getDateTime();
 		mDeferTimeLabel->setText(mDeferDateTime.isValid() ? mDeferDateTime.formatLocale() : QString());
+		contentsChanged();
 	}
 }
 
@@ -1062,7 +1227,7 @@ void EditAlarmDlg::slotShowMainPage()
 	else
 	{
 		// Set scroll position to top, since it otherwise jumps randomly
-		DialogScroll<EditAlarmDlg>* main = static_cast<DialogScroll<EditAlarmDlg>*>(mTabs->widget(0));
+		StackedScrollWidget* main = static_cast<StackedScrollWidget*>(mTabs->widget(0));
 		main->verticalScrollBar()->setValue(0);
 	}
 	if (mTimeWidget)
