@@ -1,4 +1,5 @@
-/*  functions.cpp  -  miscellaneous functions
+/*
+ *  functions.cpp  -  miscellaneous functions
  *  Program:  kalarm
  *  Copyright © 2001-2009 by David Jarvie <djarvie@kde.org>
  *
@@ -19,13 +20,16 @@
 
 #include "kalarm.h"   //krazy:exclude=includes (kalarm.h must be first)
 #include "functions.h"
+#include "functions_p.h"
 
 #include "alarmcalendar.h"
 #include "alarmevent.h"
+#include "autoqpointer.h"
 #include "eventlistmodel.h"
 #include "alarmlistview.h"
 #include "alarmresources.h"
 #include "editdlg.h"
+#include "identities.h"
 #include "kalarmapp.h"
 #include "kamail.h"
 #include "mainwindow.h"
@@ -35,10 +39,19 @@
 #include "templatelistview.h"
 #include "templatemenuaction.h"
 
+#ifdef Q_WS_X11
+#include <kwindowsystem.h>
+#include <kxmessages.h>
+#include <kstartupinfo.h>
+#include <netwm.h>
+#include <QX11Info>
+#endif
+
 #include <QDir>
 #include <QRegExp>
 #include <QDesktopWidget>
 #include <QtDBus/QtDBus>
+#include <qglobal.h>
 
 #include <kconfiggroup.h>
 #include <kaction.h>
@@ -61,7 +74,7 @@
 #include <kpimidentities/identitymanager.h>
 #include <kpimidentities/identity.h>
 #include <kcal/person.h>
-#include <libkholidays/kholidays.h>
+#include <kholidays/holidays.h>
 #include <ktoolinvocation.h>
 
 
@@ -74,13 +87,13 @@ QDBusInterface* korgInterface = 0;
 
 const char*   KMAIL_DBUS_SERVICE      = "org.kde.kmail";
 //const char*   KMAIL_DBUS_IFACE        = "org.kde.kmail.kmail";
-const char*   KMAIL_DBUS_WINDOW_PATH  = "/kmail/kmail_mainwindow_1";
+//const char*   KMAIL_DBUS_WINDOW_PATH  = "/kmail/kmail_mainwindow_1";
 const char*   KORG_DBUS_SERVICE       = "org.kde.korganizer";
 const char*   KORG_DBUS_IFACE         = "org.kde.korganizer.Korganizer";
 // D-Bus object path of KOrganizer's notification interface
 #define       KORG_DBUS_PATH            "/Korganizer"
 #define       KORG_DBUS_LOAD_PATH       "/korganizer_PimApplication"
-const char*   KORG_DBUS_WINDOW_PATH   = "/korganizer/MainWindow_1";
+//const char*   KORG_DBUS_WINDOW_PATH   = "/korganizer/MainWindow_1";
 const QString KORGANIZER_UID         = QString::fromLatin1("-korg");
 
 const char*   ALARM_OPTS_FILE        = "alarmopts";
@@ -97,7 +110,7 @@ QString uidKOrganizer(const QString& eventID);
 namespace KAlarm
 {
 
-void doEditNewAlarm(EditAlarmDlg*, bool alreadyExecuted = false);
+Private* Private::mInstance = 0;
 
 
 /******************************************************************************
@@ -146,12 +159,27 @@ KToggleAction* createAlarmEnableAction(QObject* parent)
 	return action;
 }
 
+/******************************************************************************
+* Create a "Spread Windows" action.
+*/
+KToggleAction* createSpreadWindowsAction(QObject* parent)
+{
+	KToggleAction* action = new KToggleAction(i18nc("@action", "Spread Windows"), parent);
+	QObject::connect(action, SIGNAL(triggered(bool)), theApp(), SLOT(spreadWindows(bool)));
+	// The following line ensures that all instances are kept in the same state
+	QObject::connect(theApp(), SIGNAL(spreadWindowsToggled(bool)), action, SLOT(setChecked(bool)));
+	return action;
+}
+
 /******************************************************************************
 * Create a New From Template QAction.
 */
 TemplateMenuAction* createNewFromTemplateAction(const QString& label, KActionCollection* actions, const QString& name)
 {
-	return new TemplateMenuAction(KIcon(QLatin1String("document-new-from-template")), label, actions, name);
+	TemplateMenuAction* action = new TemplateMenuAction(KIcon(QLatin1String("document-new-from-template")), label, actions, name);
+	QObject::connect(EventListModel::templates(), SIGNAL(haveEventsStatus(bool)), action, SLOT(setEnabled(bool)));
+	action->setEnabled(EventListModel::templates()->haveEvents());
+	return action;
 }
 
 /******************************************************************************
@@ -335,8 +363,6 @@ UpdateStatus addTemplate(KAEvent& event, AlarmResource* resource, QWidget* msgPa
 			status = SAVE_FAILED;
 		else
 		{
-			cal->emitEmptyStatus();
-
 			// Update the window lists
 			EventListModel::templates()->addEvent(newev);
 			return UPDATE_OK;
@@ -495,8 +521,9 @@ UpdateStatus deleteEvents(KAEvent::List& events, bool archive, QWidget* msgParen
 		KAEvent* event = events[i];
 		QString id = event->id();
 
-		// Update the window lists
+		// Update the window lists and clear stored command errors
 		EventListModel::alarms()->removeEvent(id);
+		event->setCommandError(KAEvent::CMD_NO_ERROR);
 
 		// Delete the event from the calendar file
 		if (event->category() != KCalEvent::ARCHIVED)
@@ -572,7 +599,6 @@ UpdateStatus deleteTemplates(const QStringList& eventIDs, QWidget* msgParent)
 		status = SAVE_FAILED;
 		warnErr = eventIDs.count();
 	}
-	cal->emitEmptyStatus();
 	if (status != UPDATE_OK  &&  msgParent)
 		displayUpdateError(msgParent, status, ERR_TEMPLATE, warnErr);
 	return status;
@@ -744,7 +770,7 @@ void purgeArchive(int purgeDays)
 	if (purgeDays < 0)
 		return;
 	kDebug() << purgeDays;
-	QDate cutoff = QDate::currentDate().addDays(-purgeDays);
+	QDate cutoff = KDateTime::currentLocalDate().addDays(-purgeDays);
 	AlarmResource* resource = AlarmResources::instance()->getStandardResource(AlarmResource::ARCHIVED);
 	if (!resource)
 		return;
@@ -820,12 +846,11 @@ void displayKOrgUpdateError(QWidget* parent, UpdateError code, UpdateStatus korg
 			return;
 	}
 	QString msg;
-/*TODO: uncomment after string freeze is lifted
 	if (korgError == UPDATE_KORG_ERRSTART)
 		msg = i18nc("@info", "<para>%1</para><para>(KOrganizer not fully started)</para>", errmsg);
 	else if (korgError == UPDATE_KORG_ERR)
 		msg = i18nc("@info", "<para>%1</para><para>(Error communicating with KOrganizer)</para>", errmsg);
-	else*/
+	else
 		msg = errmsg;
 	KMessageBox::error(parent, msg);
 }
@@ -835,9 +860,11 @@ void displayKOrgUpdateError(QWidget* parent, UpdateError code, UpdateStatus korg
 */
 void editNewAlarm(EditAlarmDlg::Type type, QWidget* parent)
 {
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(false, type, true, parent);
-	doEditNewAlarm(editDlg);
-	delete editDlg;
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(false, type, true, parent);
+	execNewAlarmDlg(editDlg);
 }
 
 /******************************************************************************
@@ -863,11 +890,13 @@ void editNewAlarm(KAEvent::Action action, QWidget* parent, const AlarmText* text
 		default:
 			return;
 	}
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(false, type, true, parent);
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(false, type, true, parent);
 	if (setAction  ||  text)
 		editDlg->setAction(action, *text);
-	doEditNewAlarm(editDlg);
-	delete editDlg;
+	execNewAlarmDlg(editDlg);
 }
 
 /******************************************************************************
@@ -876,15 +905,17 @@ void editNewAlarm(KAEvent::Action action, QWidget* parent, const AlarmText* text
 */
 void editNewAlarm(const KAEvent* preset, QWidget* parent)
 {
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(false, preset, true, parent);
-	doEditNewAlarm(editDlg);
-	delete editDlg;
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(false, preset, true, parent);
+	execNewAlarmDlg(editDlg);
 }
 
 /******************************************************************************
 * Common code for editNewAlarm() variants.
 */
-void doEditNewAlarm(EditAlarmDlg* editDlg, bool alreadyExecuted)
+void execNewAlarmDlg(EditAlarmDlg* editDlg, bool alreadyExecuted)
 {
 	if (alreadyExecuted  ||  editDlg->exec() == QDialog::Accepted)
 	{
@@ -959,10 +990,13 @@ void editNewTemplate(EditAlarmDlg::Type type, const KAEvent* preset, QWidget* pa
 {
 	if (!AlarmResources::instance()->activeCount(AlarmResource::TEMPLATE, true))
 	{
-		KMessageBox::sorry(parent, i18nc("@info", "You must select a resource to save the alarm in"));
+		KMessageBox::sorry(parent, i18nc("@info", "You must enable a template calendar to save the template in"));
 		return;
 	}
-	EditAlarmDlg* editDlg;
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg;
 	if (preset)
 		editDlg = EditAlarmDlg::create(true, preset, true, parent);
 	else
@@ -977,7 +1011,6 @@ void editNewTemplate(EditAlarmDlg::Type type, const KAEvent* preset, QWidget* pa
 		KAlarm::addTemplate(event, resource, editDlg);
 		Undo::saveAdd(event, resource);
 	}
-	delete editDlg;
 }
 
 } // namespace
@@ -996,14 +1029,17 @@ void editAlarm(KAEvent* event, QWidget* parent)
 		return;
 	}
 	QString id = event->id();
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(false, event, false, parent, EditAlarmDlg::RES_USE_EVENT_ID);
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(false, event, false, parent, EditAlarmDlg::RES_USE_EVENT_ID);
 	if (editDlg->exec() == QDialog::Accepted)
 	{
 		if (!AlarmCalendar::resources()->event(id))
 		{
 			// Event has been deleted while the user was editing the alarm,
 			// so treat it as a new alarm.
-			doEditNewAlarm(editDlg, true);
+			execNewAlarmDlg(editDlg, true);
 			return;
 		}
 		KAEvent newEvent;
@@ -1028,7 +1064,6 @@ void editAlarm(KAEvent* event, QWidget* parent)
 
 		outputAlarmWarnings(editDlg, &newEvent);
 	}
-	delete editDlg;
 }
 
 /******************************************************************************
@@ -1069,13 +1104,18 @@ void editTemplate(KAEvent* event, QWidget* parent)
 {
 	if (AlarmCalendar::resources()->eventReadOnly(event->id()))
 	{
-		// The template is read-only, so make the dialogue read-only
-		EditAlarmDlg* editDlg = EditAlarmDlg::create(true, event, false, parent, EditAlarmDlg::RES_PROMPT, true);
+		// The template is read-only, so make the dialogue read-only.
+		// Use AutoQPointer to guard against crash on application exit while
+		// the dialogue is still open. It prevents double deletion (both on
+		// deletion of parent, and on return from this function).
+		AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(true, event, false, parent, EditAlarmDlg::RES_PROMPT, true);
 		editDlg->exec();
-		delete editDlg;
 		return;
 	}
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(true, event, false, parent, EditAlarmDlg::RES_USE_EVENT_ID);
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(true, event, false, parent, EditAlarmDlg::RES_USE_EVENT_ID);
 	if (editDlg->exec() == QDialog::Accepted)
 	{
 		KAEvent newEvent;
@@ -1089,7 +1129,6 @@ void editTemplate(KAEvent* event, QWidget* parent)
 		updateTemplate(newEvent, editDlg);
 		Undo::saveEdit(undo, newEvent);
 	}
-	delete editDlg;
 }
 
 /******************************************************************************
@@ -1097,9 +1136,11 @@ void editTemplate(KAEvent* event, QWidget* parent)
 */
 void viewAlarm(const KAEvent* event, QWidget* parent)
 {
-	EditAlarmDlg* editDlg = EditAlarmDlg::create(false, event, false, parent, EditAlarmDlg::RES_PROMPT, true);
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<EditAlarmDlg> editDlg = EditAlarmDlg::create(false, event, false, parent, EditAlarmDlg::RES_PROMPT, true);
 	editDlg->exec();
-	delete editDlg;
 }
 
 /******************************************************************************
@@ -1186,58 +1227,113 @@ void refreshAlarmsIfQueued()
 }
 
 /******************************************************************************
-*  Start KMail if it isn't already running, and optionally iconise it.
+*  Start KMail if it isn't already running, optionally minimised.
 *  Reply = reason for failure to run KMail (which may be the empty string)
 *        = null string if success.
 */
 QString runKMail(bool minimise)
 {
-	QString dbusService = KMAIL_DBUS_SERVICE;
-	QString errmsg;
-	if (!runProgram(QLatin1String("kmail"), dbusService, (minimise ? QLatin1String(KMAIL_DBUS_WINDOW_PATH) : QString()), errmsg))
-		return i18nc("@info", "Unable to start <application>KMail</application><nl/>(<message>%1</message>)", errmsg);
+	QDBusReply<bool> reply = QDBusConnection::sessionBus().interface()->isServiceRegistered(KMAIL_DBUS_SERVICE);
+	if (!reply.isValid()  ||  !reply.value())
+	{
+		// Program is not already running, so start it
+		QString errmsg;
+		if (minimise  &&  Private::startKMailMinimised())
+			return QString();
+		if (KToolInvocation::startServiceByDesktopName(QLatin1String("kmail"), QString(), &errmsg))
+		{
+			kError() << "Couldn't start KMail (" << errmsg << ")";
+			return i18nc("@info", "Unable to start <application>KMail</application><nl/>(<message>%1</message>)", errmsg);
+		}
+	}
 	return QString();
 }
 
 /******************************************************************************
-*  Start another program for D-Bus access if it isn't already running.
-*  If 'dbusWindowPath' is not empty, the program's window with that D-Bus path
-*  name is iconised.
-*  On exit, 'errorMessage' contains an error message if failure.
-*  Reply = true if the program is now running.
+* Start KMail, minimised.
+* This code is taken from kstart in kdebase.
 */
-bool runProgram(const QString& program, QString& dbusService, const QString& dbusWindowPath, QString& errorMessage)
+bool Private::startKMailMinimised()
 {
-	QDBusReply<bool> reply = QDBusConnection::sessionBus().interface()->isServiceRegistered(dbusService);
-	if (!reply.isValid()  ||  !reply.value())
+#ifdef Q_WS_X11
+	NETRootInfo i(QX11Info::display(), NET::Supported);
+	if (i.isSupported(NET::WM2KDETemporaryRules))
 	{
-		// Program is not already running, so start it
-		if (KToolInvocation::startServiceByDesktopName(program, QString(), &errorMessage, &dbusService))
-		{
-			kError() << "Couldn't start" << program << " (" << errorMessage << ")";
-			return false;
-		}
-		if (!dbusWindowPath.isEmpty())
-		{
-			// Minimise its window - don't use hide() since this would remove all
-			// trace of it from the panel if it is not configured to be docked in
-			// the system tray.
-#ifdef __GNUC__
-#warning Make minimise window work
-#endif
-//Call D-Bus app/MainWin showMinimized()???
-#if 0
-//			QDBusInterface iface(dbusService, DBUS_PATH, DBUS_IFACE);
-//			QDBusInterface iface(dbusService, dbusWindowPath);
-			QDBusInterface iface(dbusService, "/kmail_mainwindow_1", "com.trolltech.Qt.QWidget");
-			QDBusError err = iface.callWithArgumentList(QDBus::NoBlock, QLatin1String("showMinimized"), QList<QVariant>());
-			if (err.isValid())
-				kError() << program << ": showMinimized D-Bus call failed:" << err.message();
-#endif
-		}
+		kDebug() << "using rules";
+		KXMessages msg;
+		QString message = "wmclass=kmail\nwmclassmatch=1\n"  // 1 = exact match
+		                  "wmclasscomplete=false\n"
+		                  "minimize=true\nminimizerule=3\n"
+		                  "type=" + QString().setNum(NET::Normal) + "\ntyperule=2";
+		msg.broadcastMessage("_KDE_NET_WM_TEMPORARY_RULES", message, -1, false);
+		qApp->flush();
 	}
-	errorMessage.clear();
+	else
+	{
+		// Connect to window add to get the NEW windows
+		kDebug() << "connecting to window add";
+		connect(KWindowSystem::self(), SIGNAL(windowAdded(WId)), instance(), SLOT(windowAdded(WId)));
+	}
+	// Propagate the app startup notification info to the started app.
+	// We are not using KApplication, so the env remained set.
+	KStartupInfoId id = KStartupInfo::currentStartupIdEnv();
+	KProcess* proc = new KProcess;
+	(*proc) << "kmail";
+	int pid = proc->startDetached();
+	if (!pid)
+	{
+		KStartupInfo::sendFinish(id); // failed to start
+		return false;
+	}
+	KStartupInfoData data;
+	data.addPid(pid);
+	data.setName("kmail");
+	data.setBin("kmail");
+	KStartupInfo::sendChange(id, data);
 	return true;
+#else
+	return false;
+#endif
+}
+
+/******************************************************************************
+* Called when a window is created, to minimise it.
+* This code is taken from kstart in kdebase.
+*/
+void Private::windowAdded(WId w)
+{
+#ifdef Q_WS_X11
+	static const int SUPPORTED_TYPES = NET::NormalMask | NET::DesktopMask | NET::DockMask
+	                                 | NET::ToolbarMask | NET::MenuMask | NET::DialogMask
+	                                 | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask;
+kDebug();
+	KWindowInfo kwinfo = KWindowSystem::windowInfo(w, NET::WMWindowType | NET::WMName);
+	if (kwinfo.windowType(SUPPORTED_TYPES) == NET::TopMenu
+	||  kwinfo.windowType(SUPPORTED_TYPES) == NET::Toolbar
+	||  kwinfo.windowType(SUPPORTED_TYPES) == NET::Desktop)
+		return;   // always ignore these window types
+
+kDebug()<<"1";
+	QX11Info qxinfo;
+	XWithdrawWindow(QX11Info::display(), w, qxinfo.screen());
+	QApplication::flush();
+
+	NETWinInfo info(QX11Info::display(), w, QX11Info::appRootWindow(), NET::WMState);
+	XWMHints* hints = XGetWMHints(QX11Info::display(), w);
+	if (hints)
+	{
+		hints->flags |= StateHint;
+		hints->initial_state = IconicState;
+		XSetWMHints(QX11Info::display(), w, hints);
+		XFree(hints);
+	}
+	info.setWindowType(NET::Normal);
+
+	XSync(QX11Info::display(), False);
+	XMapWindow(QX11Info::display(), w);
+	XSync(QX11Info::display(), False);
+	QApplication::flush();
+#endif
 }
 
 /******************************************************************************
@@ -1419,7 +1515,10 @@ FileType fileType(const KMimeType::Ptr& mimetype)
 *                   the user's home directory will be used. Updated to the
 *                   directory containing the selected file, if a file is chosen.
 * @param mode OR of KFile::Mode values, e.g. ExistingOnly, LocalOnly.
-* Reply = URL selected. If none is selected, URL.isEmpty() is true.
+* Reply = URL selected.
+*       = empty, non-null string if no file was selected.
+*       = null string if dialogue was deleted while visible (indicating that
+*         the parent widget was probably also deleted).
 */
 QString browseFile(const QString& caption, QString& defaultDir, const QString& initialFile,
                    const QString& filter, KFile::Modes mode, QWidget* parent)
@@ -1427,16 +1526,21 @@ QString browseFile(const QString& caption, QString& defaultDir, const QString& i
 	QString initialDir = !initialFile.isEmpty() ? QString(initialFile).remove(QRegExp("/[^/]*$"))
 	                   : !defaultDir.isEmpty()  ? defaultDir
 	                   :                          QDir::homePath();
-	KFileDialog fileDlg(initialDir, filter, parent);
-	fileDlg.setOperationMode(mode & KFile::ExistingOnly ? KFileDialog::Opening : KFileDialog::Saving);
-	fileDlg.setMode(KFile::File | mode);
-	fileDlg.setCaption(caption);
+	// Use AutoQPointer to guard against crash on application exit while
+	// the dialogue is still open. It prevents double deletion (both on
+	// deletion of parent, and on return from this function).
+	AutoQPointer<KFileDialog> fileDlg = new KFileDialog(initialDir, filter, parent);
+	fileDlg->setOperationMode(mode & KFile::ExistingOnly ? KFileDialog::Opening : KFileDialog::Saving);
+	fileDlg->setMode(KFile::File | mode);
+	fileDlg->setCaption(caption);
 	if (!initialFile.isEmpty())
-		fileDlg.setSelection(initialFile);
-	if (fileDlg.exec() != QDialog::Accepted)
-		return QString();
-	KUrl url = fileDlg.selectedUrl();
-	defaultDir = url.path();
+		fileDlg->setSelection(initialFile);
+	if (fileDlg->exec() != QDialog::Accepted)
+		return fileDlg ? QString("") : QString();
+	KUrl url = fileDlg->selectedUrl();
+	if (url.isEmpty())
+		return QString("");
+	defaultDir = url.isLocalFile() ? url.toLocalFile() : url.path();
 	return (mode & KFile::LocalOnly) ? url.pathOrUrl() : url.prettyUrl();
 }
 
@@ -1457,55 +1561,25 @@ bool isWorkingTime(const KDateTime& dt, const KAEvent* event)
 	   ||  (dt.time() >= Preferences::workDayStart()  &&  dt.time() < Preferences::workDayEnd());
 }
 
-/******************************************************************************
-*  Return the first day of the week for the user's locale.
-*  Reply = 1 (Mon) .. 7 (Sun).
-*/
-int localeFirstDayOfWeek()
-{
-	static int firstDay = 0;
-	if (!firstDay)
-		firstDay = KGlobal::locale()->weekStartDay();
-	return firstDay;
-}
-
-/******************************************************************************
-* Return the week day name (Monday = 1).
-*/
-QString weekDayName(int day, const KLocale* locale)
-{
-	switch (day)
-	{
-		case 1: return ki18nc("@option Name of the weekday", "Monday").toString(locale);
-		case 2: return ki18nc("@option Name of the weekday", "Tuesday").toString(locale);
-		case 3: return ki18nc("@option Name of the weekday", "Wednesday").toString(locale);
-		case 4: return ki18nc("@option Name of the weekday", "Thursday").toString(locale);
-		case 5: return ki18nc("@option Name of the weekday", "Friday").toString(locale);
-		case 6: return ki18nc("@option Name of the weekday", "Saturday").toString(locale);
-		case 7: return ki18nc("@option Name of the weekday", "Sunday").toString(locale);
-	}
-	return QString();
-}
-
 /******************************************************************************
 * Convert a date/time specification string into a local date/time or date value.
 * Parameters:
-*   tzString  = in the form [[[yyyy-]mm-]dd-]hh:mm [TZ] or yyyy-mm-dd [TZ].
+*   timeString  = in the form [[[yyyy-]mm-]dd-]hh:mm [TZ] or yyyy-mm-dd [TZ].
 *   dateTime  = receives converted date/time value.
-*   defaultDt = default date/time used for missing parts of tzString, or null
+*   defaultDt = default date/time used for missing parts of timeString, or null
 *               to use current date/time.
-*   allowTZ   = whether to allow a time zone specifier in tzString.
+*   allowTZ   = whether to allow a time zone specifier in timeString.
 * Reply = true if successful.
 */
-bool convTimeString(const QByteArray& tzString, KDateTime& dateTime, const KDateTime& defaultDt, bool allowTZ)
+bool convertTimeString(const QByteArray& timeString, KDateTime& dateTime, const KDateTime& defaultDt, bool allowTZ)
 {
 #define MAX_DT_LEN 19
-	int i = tzString.indexOf(' ');
+	int i = timeString.indexOf(' ');
 	if (i > MAX_DT_LEN  ||  (i >= 0 && !allowTZ))
 		return false;
-	QString zone = (i >= 0) ? QString::fromLatin1(tzString.mid(i)) : QString();
+	QString zone = (i >= 0) ? QString::fromLatin1(timeString.mid(i)) : QString();
 	char timeStr[MAX_DT_LEN+1];
-	strcpy(timeStr, tzString.left(i >= 0 ? i : MAX_DT_LEN));
+	strcpy(timeStr, timeString.left(i >= 0 ? i : MAX_DT_LEN));
 	int dt[5] = { -1, -1, -1, -1, -1 };
 	char* s;
 	char* end;
@@ -1625,15 +1699,7 @@ KDateTime applyTimeZone(const QString& tzstring, const QDate& date, const QTime&
 	bool error = false;
 	KDateTime::Spec spec = KDateTime::LocalZone;
 	QString zone = tzstring.trimmed();
-	if (defaultDt.isValid())
-	{
-		// Time spec is supplied - time zone specifier is not allowed
-		if (!zone.isEmpty())
-			error = true;
-		else
-			spec = defaultDt.timeSpec();
-	}
-	else if (!zone.isEmpty())
+	if (!zone.isEmpty())
 	{
 		if (zone == QLatin1String("Clock"))
 			spec = KDateTime::ClockTime;
@@ -1647,6 +1713,8 @@ KDateTime applyTimeZone(const QString& tzstring, const QDate& date, const QTime&
 				spec = tz;
 		}
 	}
+	else if (defaultDt.isValid())
+		spec = defaultDt.timeSpec();
 
 	KDateTime result;
 	if (!error)
@@ -1657,7 +1725,7 @@ KDateTime applyTimeZone(const QString& tzstring, const QDate& date, const QTime&
 			if (defaultDt.isValid())
 			       result = KDateTime(defaultDt.date(), time, spec);
 			else if (spec == KDateTime::LocalZone  ||  spec == KDateTime::ClockTime)
-				result = KDateTime(QDate::currentDate(), time, spec);
+				result = KDateTime(KDateTime::currentLocalDate(), time, spec);
 		}
 		else if (haveTime)
 		{
@@ -1673,6 +1741,32 @@ KDateTime applyTimeZone(const QString& tzstring, const QDate& date, const QTime&
 	return result;
 }
 
+#ifndef NDEBUG
+/******************************************************************************
+* Set up KAlarm test conditions based on environment variables.
+* KALARM_TIME: specifies current system time (format [[[yyyy-]mm-]dd-]hh:mm [TZ]).
+*/
+void setTestModeConditions()
+{
+	const QByteArray newTime = qgetenv("KALARM_TIME");
+	if (!newTime.isEmpty())
+	{
+		KDateTime dt;
+		if (convertTimeString(newTime, dt, KDateTime::realCurrentLocalDateTime(), true))
+			setSimulatedSystemTime(dt);
+	}
+}
+
+/******************************************************************************
+* Set the simulated system time.
+*/
+void setSimulatedSystemTime(const KDateTime& dt)
+{
+	KDateTime::setSimulatedSystemTime(dt);
+	kDebug() << "New time =" << qPrintable(KDateTime::currentLocalDateTime().toString("%Y-%m-%d %H:%M %:Z"));
+}
+#endif
+
 } // namespace KAlarm
 
 
@@ -1702,7 +1796,7 @@ KAlarm::UpdateStatus sendToKOrganizer(const KAEvent* event)
 		case KAEvent::EMAIL:
 		{
 			QString from = event->emailFromId()
-			             ? KAMail::identityManager()->identityForUoid(event->emailFromId()).fullEmailAddr()
+			             ? Identities::identityManager()->identityForUoid(event->emailFromId()).fullEmailAddr()
 			             : Preferences::emailAddress();
 			AlarmText atext;
 			atext.setEmail(event->emailAddresses(", "), from, QString(), QString(), event->emailSubject(), QString());
