@@ -27,9 +27,6 @@
  *  you do not wish to do so, delete this exception statement from
  *  your version.
  */
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #include "folderjob.h"
 
@@ -38,25 +35,26 @@
 #include "folderstorage.h"
 
 #include <kdebug.h>
+#include <kio/global.h>
 
 namespace KMail {
 
 //----------------------------------------------------------------------------
 FolderJob::FolderJob( KMMessage *msg, JobType jt, KMFolder* folder,
-                          QString partSpecifier )
+                          const QString &partSpecifier )
   : mType( jt ), mSrcFolder( 0 ), mDestFolder( folder ), mPartSpecifier( partSpecifier ),
     mErrorCode( 0 ),
     mPassiveDestructor( false ), mStarted( false )
 {
   if ( msg ) {
-    mMsgList.append(msg);
-    mSets = msg->headerField("X-UID");
+    mMsgList.append( msg );
+    mSets = msg->headerField( "X-UID", KMMessage::NoEncoding );
   }
   init();
 }
 
 //----------------------------------------------------------------------------
-FolderJob::FolderJob( const QPtrList<KMMessage>& msgList, const QString& sets,
+FolderJob::FolderJob( const QList<KMMessage*>& msgList, const QString& sets,
                           JobType jt, KMFolder *folder )
   : mMsgList( msgList ),mType( jt ),
     mSets( sets ), mSrcFolder( 0 ), mDestFolder( folder ),
@@ -100,8 +98,7 @@ FolderJob::~FolderJob()
 }
 
 //----------------------------------------------------------------------------
-void
-FolderJob::start()
+void FolderJob::start()
 {
   if (!mStarted)
   {
@@ -113,12 +110,12 @@ FolderJob::start()
 //----------------------------------------------------------------------------
 void FolderJob::kill()
 {
+  mErrorCode = KIO::ERR_USER_CANCELED;
   delete this;
 }
 
 //----------------------------------------------------------------------------
-QPtrList<KMMessage>
-FolderJob::msgList() const
+QList<KMMessage*> FolderJob::msgList() const
 {
   return mMsgList;
 }
