@@ -17,11 +17,11 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
+// my header
 #include "kmfilter.h"
+
+// other kmail headers
 #include "kmkernel.h"
 #include "accountmanager.h"
 using KMail::AccountManager;
@@ -31,23 +31,21 @@ using KMail::AccountManager;
 #include "filterlog.h"
 using KMail::FilterLog;
 
+// other KDE headers
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
 
+// other headers
 #include <assert.h>
 
 
-KMFilter::KMFilter( KConfig* aConfig, bool popFilter )
+KMFilter::KMFilter( bool popFilter )
   : bPopFilter(popFilter)
 {
-  if (!bPopFilter)
-    mActions.setAutoDelete( true );
-
-  if ( aConfig )
-    readConfig( aConfig );
-  else if ( bPopFilter )
+  if ( bPopFilter )
     mAction = Down;
   else {
     bApplyOnInbound = true;
@@ -62,13 +60,17 @@ KMFilter::KMFilter( KConfig* aConfig, bool popFilter )
 }
 
 
+KMFilter::KMFilter( KConfigGroup & aConfig, bool popFilter )
+  : bPopFilter(popFilter)
+{
+  readConfig( aConfig );
+}
+
+
 KMFilter::KMFilter( const KMFilter & aFilter )
 {
   bPopFilter = aFilter.isPopFilter();
 
-  if ( !bPopFilter )
-    mActions.setAutoDelete( true );
-
   mPattern = aFilter.mPattern;
 
   if ( bPopFilter ){
@@ -80,40 +82,49 @@ KMFilter::KMFilter( const KMFilter & aFilter )
     bStopProcessingHere = aFilter.stopProcessingHere();
     bConfigureShortcut = aFilter.configureShortcut();
     bConfigureToolbar = aFilter.configureToolbar();
+    mToolbarName = aFilter.toolbarName();
     mApplicability = aFilter.applicability();
     mIcon = aFilter.icon();
     mShortcut = aFilter.shortcut();
 
-    QPtrListIterator<KMFilterAction> it( aFilter.mActions );
-    for ( it.toFirst() ; it.current() ; ++it ) {
-      KMFilterActionDesc *desc = (*kmkernel->filterActionDict())[ (*it)->name() ];
+    QListIterator<KMFilterAction*> it( aFilter.mActions );
+    while ( it.hasNext() ) {
+      KMFilterAction *action = it.next();
+      KMFilterActionDesc *desc = kmkernel->filterActionDict()->value( action->name() );
       if ( desc ) {
         KMFilterAction *f = desc->create();
         if ( f ) {
-          f->argsFromString( (*it)->argsAsString() );
+          f->argsFromString( action->argsAsString() );
           mActions.append( f );
         }
       }
     }
 
     mAccounts.clear();
-    QValueListConstIterator<int> it2;
+    QList<int>::ConstIterator it2;
     for ( it2 = aFilter.mAccounts.begin() ; it2 != aFilter.mAccounts.end() ; ++it2 )
       mAccounts.append( *it2 );
   }
 }
 
+KMFilter::~KMFilter()
+{
+  if ( !bPopFilter ) {
+    qDeleteAll( mActions );
+  }
+}
+
 // only for !bPopFilter
 KMFilter::ReturnCode KMFilter::execActions( KMMessage* msg, bool& stopIt ) const
 {
   ReturnCode status = NoResult;
 
-  QPtrListIterator<KMFilterAction> it( mActions );
-  for ( it.toFirst() ; it.current() ; ++it ) {
+  QList<KMFilterAction*>::const_iterator it( mActions.begin() );
+  for ( ; it != mActions.end() ; ++it ) {
 
     if ( FilterLog::instance()->isLogging() ) {
-      QString logText( i18n( "<b>Applying filter action:</b> %1" )
-                       .arg( (*it)->displayString() ) );
+      QString logText( i18n( "<b>Applying filter action:</b> %1",
+                         (*it)->displayString() ) );
       FilterLog::instance()->add( logText, FilterLog::appliedAction );
     }
 
@@ -151,9 +162,9 @@ bool KMFilter::requiresBody( KMMsgBase* msg )
 {
   if (pattern() && pattern()->requiresBody())
     return true; // no pattern means always matches?
-  QPtrListIterator<KMFilterAction> it( *actions() );
-  for ( it.toFirst() ; it.current() ; ++it )
-    if ((*it)->requiresBody( msg ))
+  QListIterator<KMFilterAction*> it( *actions() );
+  while ( it.hasNext() )
+    if ( it.next()->requiresBody( msg ))
       return true;
   return false;
 }
@@ -176,9 +187,9 @@ bool KMFilter::folderRemoved( KMFolder* aFolder, KMFolder* aNewFolder )
 {
   bool rem = false;
 
-  QPtrListIterator<KMFilterAction> it( mActions );
-  for ( it.toFirst() ; it.current() ; ++it )
-    if ( (*it)->folderRemoved( aFolder, aNewFolder ) )
+  QListIterator<KMFilterAction*> it( mActions );
+  while ( it.hasNext() )
+    if ( it.next()->folderRemoved( aFolder, aNewFolder ) )
       rem = true;
 
   return rem;
@@ -189,7 +200,7 @@ void KMFilter::setApplyOnAccount( uint id, bool aApply )
   if (aApply && !mAccounts.contains( id )) {
     mAccounts.append( id );
   } else if (!aApply && mAccounts.contains( id )) {
-    mAccounts.remove( id );
+    mAccounts.removeAll( id );
   }
 }
 
@@ -210,7 +221,7 @@ bool KMFilter::applyOnAccount( uint id ) const
 
 
 //-----------------------------------------------------------------------------
-void KMFilter::readConfig(KConfig* config)
+void KMFilter::readConfig(KConfigGroup & config)
 {
   // MKSearchPattern::readConfig ensures
   // that the pattern is purified.
@@ -218,7 +229,7 @@ void KMFilter::readConfig(KConfig* config)
 
   if (bPopFilter) {
     // get the action description...
-    QString action = config->readEntry( "action" );
+    QString action = config.readEntry( "action" );
     if ( action == "down" )
       mAction = Down;
     else if ( action == "later" )
@@ -229,8 +240,8 @@ void KMFilter::readConfig(KConfig* config)
       mAction = NoAction;
   }
   else {
-    QStringList sets = config->readListEntry("apply-on");
-    if ( sets.isEmpty() && !config->hasKey("apply-on") ) {
+    QStringList sets = config.readEntry("apply-on", QStringList() );
+    if ( sets.isEmpty() && !config.hasKey("apply-on") ) {
       bApplyOnOutbound = false;
       bApplyOnInbound = true;
       bApplyOnExplicit = true;
@@ -239,43 +250,45 @@ void KMFilter::readConfig(KConfig* config)
       bApplyOnInbound = bool(sets.contains("check-mail"));
       bApplyOnOutbound = bool(sets.contains("send-mail"));
       bApplyOnExplicit = bool(sets.contains("manual-filtering"));
-      mApplicability = (AccountType)config->readNumEntry( "Applicability", ButImap );
+      mApplicability = (AccountType) config.readEntry(
+            "Applicability", (int)ButImap );
     }
 
-    bStopProcessingHere = config->readBoolEntry("StopProcessingHere", true);
-    bConfigureShortcut = config->readBoolEntry("ConfigureShortcut", false);
-    QString shortcut( config->readEntry( "Shortcut" ) );
+    bStopProcessingHere = config.readEntry( "StopProcessingHere", true );
+    bConfigureShortcut = config.readEntry( "ConfigureShortcut", false );
+    QString shortcut( config.readEntry( "Shortcut", QString() ) );
     if ( !shortcut.isEmpty() ) {
       KShortcut sc( shortcut );
       setShortcut( sc );
     }
-    bConfigureToolbar = config->readBoolEntry("ConfigureToolbar", false);
+    bConfigureToolbar = config.readEntry( "ConfigureToolbar", false );
     bConfigureToolbar = bConfigureToolbar && bConfigureShortcut;
-    mIcon = config->readEntry( "Icon", "gear" );
-    bAutoNaming = config->readBoolEntry("AutomaticName", false);
+    mToolbarName = config.readEntry( "ToolbarName", name() );
+    mIcon = config.readEntry( "Icon", "system-run" );
+    bAutoNaming = config.readEntry( "AutomaticName", false );
 
-    int i, numActions;
     QString actName, argsName;
 
     mActions.clear();
 
-    numActions = config->readNumEntry("actions",0);
+    int numActions = config.readEntry( "actions", 0 );
     if (numActions > FILTER_MAX_ACTIONS) {
       numActions = FILTER_MAX_ACTIONS ;
-      KMessageBox::information( 0, i18n("<qt>Too many filter actions in filter rule <b>%1</b>.</qt>").arg( mPattern.name() ) );
+      KMessageBox::information( 0, i18n("<qt>Too many filter actions in filter rule <b>%1</b>.</qt>", mPattern.name() ) );
     }
 
-    for ( i=0 ; i < numActions ; i++ ) {
+    for ( int i=0 ; i < numActions ; i++ ) {
       actName.sprintf("action-name-%d", i);
       argsName.sprintf("action-args-%d", i);
       // get the action description...
-      KMFilterActionDesc *desc = (*kmkernel->filterActionDict())[ config->readEntry( actName ) ];
+      KMFilterActionDesc *desc = kmkernel->filterActionDict()->value(
+            config.readEntry( actName, QString() ) );
       if ( desc ) {
         //...create an instance...
         KMFilterAction *fa = desc->create();
         if ( fa ) {
           //...load it with it's parameter...
-          fa->argsFromString( config->readEntry( argsName ) );
+          fa->argsFromString( config.readEntry( argsName, QString() ) );
           //...check if it's emoty and...
           if ( !fa->isEmpty() )
             //...append it if it's not and...
@@ -286,32 +299,33 @@ void KMFilter::readConfig(KConfig* config)
         }
       } else
         KMessageBox::information( 0 /* app-global modal dialog box */,
-                                  i18n("<qt>Unknown filter action <b>%1</b><br>in filter rule <b>%2</b>.<br>Ignoring it.</qt>")
-                                       .arg( config->readEntry( actName ) ).arg( mPattern.name() ) );
+            i18n("<qt>Unknown filter action <b>%1</b><br />in filter rule <b>%2</b>.<br />Ignoring it.</qt>",
+              config.readEntry( actName, QString() ),
+              mPattern.name() ) );
     }
 
-    mAccounts = config->readIntListEntry( "accounts-set" );
+    mAccounts = config.readEntry( "accounts-set",QList<int>() );
   }
 }
 
 
-void KMFilter::writeConfig(KConfig* config) const
+void KMFilter::writeConfig(KConfigGroup & config) const
 {
   mPattern.writeConfig(config);
 
   if (bPopFilter) {
     switch ( mAction ) {
     case Down:
-      config->writeEntry( "action", "down" );
+      config.writeEntry( "action", "down" );
       break;
     case Later:
-      config->writeEntry( "action", "later" );
+      config.writeEntry( "action", "later" );
       break;
     case Delete:
-      config->writeEntry( "action", "delete" );
+      config.writeEntry( "action", "delete" );
       break;
     default:
-      config->writeEntry( "action", "" );
+      config.writeEntry( "action", "" );
     }
   } else {
     QStringList sets;
@@ -321,29 +335,30 @@ void KMFilter::writeConfig(KConfig* config) const
       sets.append( "send-mail" );
     if ( bApplyOnExplicit )
       sets.append( "manual-filtering" );
-    config->writeEntry( "apply-on", sets );
+    config.writeEntry( "apply-on", sets );
 
-    config->writeEntry( "StopProcessingHere", bStopProcessingHere );
-    config->writeEntry( "ConfigureShortcut", bConfigureShortcut );
-    if ( !mShortcut.isNull() )
-      config->writeEntry( "Shortcut", mShortcut.toString() );
-    config->writeEntry( "ConfigureToolbar", bConfigureToolbar );
-    config->writeEntry( "Icon", mIcon );
-    config->writeEntry( "AutomaticName", bAutoNaming );
-    config->writeEntry( "Applicability", mApplicability );
+    config.writeEntry( "StopProcessingHere", bStopProcessingHere );
+    config.writeEntry( "ConfigureShortcut", bConfigureShortcut );
+    if ( !mShortcut.isEmpty() )
+      config.writeEntry( "Shortcut", mShortcut.toString() );
+    config.writeEntry( "ConfigureToolbar", bConfigureToolbar );
+    config.writeEntry( "ToolbarName", mToolbarName );
+    config.writeEntry( "Icon", mIcon );
+    config.writeEntry( "AutomaticName", bAutoNaming );
+    config.writeEntry( "Applicability", (int)mApplicability );
 
     QString key;
     int i;
 
-    QPtrListIterator<KMFilterAction> it( mActions );
-    for ( i=0, it.toFirst() ; it.current() ; ++it, ++i ) {
-      config->writeEntry( key.sprintf("action-name-%d", i),
+    QList<KMFilterAction*>::const_iterator it;
+    for ( i=0, it = mActions.begin() ; it != mActions.end() ; ++it, ++i ) {
+      config.writeEntry( key.sprintf("action-name-%d", i),
                           (*it)->name() );
-      config->writeEntry( key.sprintf("action-args-%d", i),
+      config.writeEntry( key.sprintf("action-args-%d", i),
                           (*it)->argsAsString() );
     }
-    config->writeEntry( "actions", i );
-    config->writeEntry( "accounts-set", mAccounts );
+    config.writeEntry( "actions", i );
+    config.writeEntry( "accounts-set", mAccounts );
   }
 }
 
@@ -352,19 +367,19 @@ void KMFilter::purify()
   mPattern.purify();
 
   if (!bPopFilter) {
-    QPtrListIterator<KMFilterAction> it( mActions );
-    it.toLast();
-    while ( it.current() )
-      if ( (*it)->isEmpty() )
-        mActions.remove ( (*it) );
-      else
-        --it;
+    QListIterator<KMFilterAction*> it( mActions );
+    it.toBack();
+    while ( it.hasPrevious() ) {
+      KMFilterAction *action = it.previous();
+      if ( action->isEmpty() )
+        mActions.removeAll ( action );
+    }
 
     // Remove invalid accounts from mAccounts - just to be tidy
-    QValueListConstIterator<int> it2 = mAccounts.begin();
+    QList<int>::Iterator it2 = mAccounts.begin();
     while ( it2 != mAccounts.end() ) {
       if ( !kmkernel->acctMgr()->find( *it2 ) )
-         mAccounts.remove( *it2 );
+         it2 = mAccounts.erase( it2 );
       else
          ++it2;
     }
@@ -376,7 +391,16 @@ bool KMFilter::isEmpty() const
   if (bPopFilter)
     return mPattern.isEmpty();
   else
-    return mPattern.isEmpty() && mActions.isEmpty() && mAccounts.isEmpty();
+    return ( mPattern.isEmpty() && mActions.isEmpty() ) ||
+        ( ( applicability() == Checked ) && mAccounts.isEmpty() );
+}
+
+QString KMFilter::toolbarName() const
+{
+  if ( mToolbarName.isEmpty() )
+    return name();
+  else
+    return mToolbarName;
 }
 
 #ifndef NDEBUG
@@ -384,21 +408,22 @@ const QString KMFilter::asString() const
 {
   QString result;
 
-  result += mPattern.asString();
+  result += "Filter name: " + name() + '\n';
+  result += mPattern.asString() + '\n';
 
   if (bPopFilter){
     result += "    action: ";
     result += mAction;
-    result += "\n";
+    result += '\n';
   }
   else {
-    QPtrListIterator<KMFilterAction> it( mActions );
-    for ( it.toFirst() ; it.current() ; ++it ) {
+    QList<KMFilterAction*>::const_iterator it( mActions.begin() );
+    for ( ; it != mActions.end() ; ++it ) {
       result += "    action: ";
       result += (*it)->label();
-      result += " ";
+      result += ' ';
       result += (*it)->argsAsString();
-      result += "\n";
+      result += '\n';
     }
     result += "This filter belongs to the following sets:";
     if ( bApplyOnInbound )
@@ -407,20 +432,20 @@ const QString KMFilter::asString() const
       result += " Outbound";
     if ( bApplyOnExplicit )
       result += " Explicit";
-    result += "\n";
+    result += '\n';
     if ( bApplyOnInbound && mApplicability == All ) {
       result += "This filter applies to all accounts.\n";
     } else if ( bApplyOnInbound && mApplicability == ButImap ) {
       result += "This filter applies to all but online IMAP accounts.\n";
     } else if ( bApplyOnInbound ) {
-      QValueListConstIterator<int> it2;
+      QList<int>::ConstIterator it2;
       result += "This filter applies to the following accounts:";
       if ( mAccounts.isEmpty() )
         result += " None";
       else for ( it2 = mAccounts.begin() ; it2 != mAccounts.end() ; ++it2 )
         if ( kmkernel->acctMgr()->find( *it2 ) )
-          result += " " + kmkernel->acctMgr()->find( *it2 )->name();
-      result += "\n";
+          result += ' ' + kmkernel->acctMgr()->find( *it2 )->name();
+      result += '\n';
     }
     if ( bStopProcessingHere )
       result += "If it matches, processing stops at this filter.\n";
