@@ -22,6 +22,9 @@
 // Could be extended to include support for normal main window
 // widgets like a toolbar.
 
+#include "kmreadermainwin.h"
+#include "kmreaderwin.h"
+
 #include <kicon.h>
 #include <kactionmenu.h>
 #include <kedittoolbar.h>
@@ -40,16 +43,16 @@
 #include "kmcommands.h"
 #include "kmenubar.h"
 #include "kmenu.h"
-#include "kmreaderwin.h"
 #include "kmfolder.h"
 #include "kmmainwidget.h"
-#include "kmfoldertree.h"
 #include "csshelper.h"
 #include "customtemplatesmenu.h"
 #include "messageactions.h"
 #include "kmmsgdict.h"
+#include "mainfolderview.h"
 
-#include "kmreadermainwin.h"
+#include <kabc/stdaddressbook.h>
+#include <kpimutils/email.h>
 
 KMReaderMainWin::KMReaderMainWin( bool htmlOverride, bool htmlLoadExtOverride,
                                   char *name )
@@ -97,6 +100,7 @@ void KMReaderMainWin::initKMReaderMainWin() {
   setCentralWidget( mReaderWin );
   setupAccel();
   setupGUI( Keys | StatusBar | Create, "kmreadermainwin.rc" );
+  mMsgActions->setupForwardingActionsList( this );
   applyMainWindowSettings( KMKernel::config()->group( "Separate Reader Window" ) );
   if( ! mReaderWin->message() ) {
     menuBar()->hide();
@@ -120,10 +124,15 @@ void KMReaderMainWin::setUseFixedFont( bool useFixedFont )
 }
 
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::showMsg( const QString & encoding, KMMessage *msg )
+void KMReaderMainWin::showMsg( const QString & encoding, KMMessage *msg,
+                               unsigned long serNumOfOriginalMessage, int nodeIdOffset )
 {
   mReaderWin->setOverrideEncoding( encoding );
   mReaderWin->setMsg( msg, true );
+  if ( serNumOfOriginalMessage != 0 ) {
+    Q_ASSERT( nodeIdOffset != -1 );
+    mReaderWin->setOriginalMsg( serNumOfOriginalMessage, nodeIdOffset );
+  }
   mReaderWin->slotTouchMessage();
   setCaption( msg->subject() );
   mMsg = msg;
@@ -156,7 +165,7 @@ void KMReaderMainWin::slotTrashMsg()
     KMMessage *msg = parent->getMsg( index );
     if (msg) {
       // now delete the msg and close this window
-      KMDeleteMsgCommand *command = new KMDeleteMsgCommand( parent, msg );
+      KMTrashMsgCommand *command = new KMTrashMsgCommand( parent, msg );
       command->start();
       close();
     }
@@ -175,7 +184,7 @@ void KMReaderMainWin::slotPrintMsg()
 }
 
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::slotForwardMsg()
+void KMReaderMainWin::slotForwardInlineMsg()
 {
    KMCommand *command = 0;
    if ( mReaderWin->message() && mReaderWin->message()->parent() ) {
@@ -210,7 +219,7 @@ void KMReaderMainWin::slotRedirectMsg()
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::slotCustomReplyToMsg( const QString &tmpl )
 {
-  kDebug(5006) << "Reply with template:" << tmpl;
+  kDebug() << "Reply with template:" << tmpl;
   KMCommand *command = new KMCustomReplyToCommand( this,
                                                    mReaderWin->message(),
                                                    mReaderWin->copyText(),
@@ -221,7 +230,7 @@ void KMReaderMainWin::slotCustomReplyToMsg( const QString &tmpl )
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::slotCustomReplyAllToMsg( const QString &tmpl )
 {
-  kDebug(5006) << "Reply to All with template:" << tmpl;
+  kDebug() << "Reply to All with template:" << tmpl;
   KMCommand *command = new KMCustomReplyAllToCommand( this,
                                                       mReaderWin->message(),
                                                       mReaderWin->copyText(),
@@ -232,7 +241,7 @@ void KMReaderMainWin::slotCustomReplyAllToMsg( const QString &tmpl )
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::slotCustomForwardMsg( const QString &tmpl)
 {
-  kDebug(5006) << "Forward with template:" << tmpl;
+  kDebug() << "Forward with template:" << tmpl;
   KMCommand *command = new KMCustomForwardCommand( this,
                                                    mReaderWin->message(),
                                                    0, tmpl );
@@ -254,6 +263,8 @@ void KMReaderMainWin::slotShowMsgSrc()
 void KMReaderMainWin::slotConfigChanged()
 {
   //readConfig();
+  mMsgActions->setupForwardActions();
+  mMsgActions->setupForwardingActionsList( this );
 }
 
 void KMReaderMainWin::setupAccel()
@@ -274,8 +285,8 @@ void KMReaderMainWin::setupAccel()
   connect( mSaveAtmAction, SIGNAL(triggered(bool)), mReaderWin, SLOT(slotSaveAttachments()) );
 
   mTrashAction = new KAction( KIcon( "user-trash" ), i18n("&Move to Trash"), this );
-  mTrashAction->setIconText( i18n( "Trash" ) );
-  mTrashAction->setToolTip( i18n( "Move message to trashcan" ) );
+  mTrashAction->setIconText( i18nc( "@action:intoolbar Move to Trash", "Trash" ) );
+  mTrashAction->setHelpText( i18n( "Move message to trashcan" ) );
   mTrashAction->setShortcut( QKeySequence( Qt::Key_Delete ) );
   actionCollection()->addAction( "move_to_trash", mTrashAction );
   connect( mTrashAction, SIGNAL(triggered()), this, SLOT(slotTrashMsg()) );
@@ -292,28 +303,6 @@ void KMReaderMainWin::setupAccel()
   mViewSourceAction->setShortcut(QKeySequence(Qt::Key_V));
 
   //----- Message Menu
-  mForwardActionMenu  = new KActionMenu(KIcon("mail-forward"), i18nc("Message->","&Forward"), this);
-  actionCollection()->addAction("message_forward", mForwardActionMenu );
-  connect( mForwardActionMenu, SIGNAL( activated() ), this,
-           SLOT( slotForwardMsg() ) );
-
-  mForwardAttachedAction  = new KAction(KIcon("mail-forward"), i18nc("Message->Forward->","As &Attachment..."), this);
-  actionCollection()->addAction("message_forward_as_attachment", mForwardAttachedAction );
-  mForwardAttachedAction->setShortcut(QKeySequence(Qt::Key_F));
-  connect(mForwardAttachedAction, SIGNAL(triggered(bool) ), SLOT(slotForwardAttachedMsg()));
-  mForwardActionMenu->addAction( mForwardAttachedAction );
-
-  mForwardAction  = new KAction(KIcon("mail-forward"), i18n("&Inline..."), this);
-  actionCollection()->addAction("message_forward_inline", mForwardAction );
-  connect(mForwardAction, SIGNAL(triggered(bool) ), SLOT(slotForwardMsg()));
-  mForwardAction->setShortcut(QKeySequence(Qt::SHIFT+Qt::Key_F));
-  mForwardActionMenu->addAction( mForwardAction );
-
-  mRedirectAction  = new KAction(i18nc("Message->Forward->", "&Redirect..."), this);
-  actionCollection()->addAction("message_forward_redirect", mRedirectAction );
-  connect(mRedirectAction, SIGNAL(triggered(bool)), SLOT(slotRedirectMsg()));
-  mRedirectAction->setShortcut(QKeySequence(Qt::Key_E));
-  mForwardActionMenu->addAction( mRedirectAction );
 
   fontAction = new KFontAction( i18n("Select Font"), this );
   actionCollection()->addAction( "text_font", fontAction );
@@ -332,9 +321,6 @@ void KMReaderMainWin::setupAccel()
   updateMessageMenu();
   updateCustomTemplateMenus();
 
-  mCopyTextAction = new KAction( KStandardAction::copy(
-                   mReaderWin, SLOT( slotCopySelectedText() ), actionCollection() ) );
-
   connect( mReaderWin, SIGNAL(popupMenu(KMMessage&,const KUrl&,const QPoint&)),
            this, SLOT(slotMsgPopup(KMMessage&,const KUrl&,const QPoint&)) );
   connect( mReaderWin, SIGNAL(urlClicked(const KUrl&,int)),
@@ -358,8 +344,8 @@ void KMReaderMainWin::updateCustomTemplateMenus()
              this, SLOT(slotCustomForwardMsg( const QString& )) );
   }
 
-  mForwardActionMenu->addSeparator();
-  mForwardActionMenu->addAction( mCustomTemplateMenus->forwardActionMenu() );
+  mMsgActions->forwardMenu()->addSeparator();
+  mMsgActions->forwardMenu()->addAction( mCustomTemplateMenus->forwardActionMenu() );
 
   mMsgActions->replyMenu()->addSeparator();
   mMsgActions->replyMenu()->addAction( mCustomTemplateMenus->replyActionMenu() );
@@ -370,26 +356,23 @@ void KMReaderMainWin::updateCustomTemplateMenus()
 //-----------------------------------------------------------------------------
 void KMReaderMainWin::updateMessageMenu()
 {
-  mMenuToFolder.clear();
-
   KMMainWidget* mainwin = kmkernel->getKMMainWidget();
   if ( mainwin )
-    mainwin->folderTree()->folderToPopupMenu( KMFolderTree::CopyMessage, this,
-                                              &mMenuToFolder, mCopyActionMenu->menu() );
+    mainwin->mainFolderView()->folderToPopupMenu( KMail::MainFolderView::CopyMessage, this,
+                                                  mCopyActionMenu->menu() );
 }
 
 
 //-----------------------------------------------------------------------------
-void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KUrl &aUrl, const QPoint& aPoint)
+void KMReaderMainWin::slotMsgPopup( KMMessage &aMsg, const KUrl &aUrl, const QPoint &aPoint )
 {
-  KMenu * menu = new KMenu;
+  KMenu *menu = new KMenu;
   mUrl = aUrl;
   mMsg = &aMsg;
-  bool urlMenuAdded=false;
-
-  if (!aUrl.isEmpty())
-  {
-    if (aUrl.protocol() == "mailto") {
+  bool urlMenuAdded = false;
+  bool copyAdded = false;
+  if ( !aUrl.isEmpty() ) {
+    if ( aUrl.protocol() == "mailto" ) {
       // popup on a mailto URL
       menu->addAction( mReaderWin->mailToComposeAction() );
       if ( mMsg ) {
@@ -397,9 +380,17 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KUrl &aUrl, const QPoi
         menu->addAction( mReaderWin->mailToForwardAction() );
         menu->addSeparator();
       }
-      menu->addAction( mReaderWin->addAddrBookAction() );
-      menu->addAction( mReaderWin->openAddrBookAction() );
+      QString email =  KPIMUtils::firstEmailAddress( aUrl.path() );
+      KABC::AddressBook *addressBook = KABC::StdAddressBook::self( true );
+      KABC::Addressee::List addresseeList = addressBook->findByEmail( email );
+
+      if ( addresseeList.count() == 0 ) {
+        menu->addAction( mReaderWin->addAddrBookAction() );
+      } else {
+        menu->addAction( mReaderWin->openAddrBookAction() );
+      }
       menu->addAction( mReaderWin->copyAction() );
+      copyAdded = true;
     } else {
       // popup on a not-mailto URL
       menu->addAction( mReaderWin->urlOpenAction() );
@@ -407,22 +398,22 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KUrl &aUrl, const QPoi
       menu->addAction( mReaderWin->urlSaveAsAction() );
       menu->addAction( mReaderWin->copyURLAction() );
     }
-    urlMenuAdded=true;
+    urlMenuAdded = true;
   }
-  if(!mReaderWin->copyText().isEmpty()) {
-    if ( urlMenuAdded )
+  if ( !mReaderWin->copyText().isEmpty() ) {
+    if ( urlMenuAdded ) {
       menu->addSeparator();
+    }
     menu->addAction( mMsgActions->replyMenu() );
     menu->addSeparator();
-
-    menu->addAction( mReaderWin->copyAction() );
+    if( !copyAdded )
+      menu->addAction( mReaderWin->copyAction() );
     menu->addAction( mReaderWin->selectAllAction() );
-  } else if ( !urlMenuAdded )
-  {
+  } else if ( !urlMenuAdded ) {
     // popup somewhere else (i.e., not a URL) on the message
 
-    if (!mMsg) // no message
-    {
+    if (!mMsg) {
+      // no message
       delete menu;
       return;
     }
@@ -436,7 +427,7 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KUrl &aUrl, const QPoi
       // FIXME: needs custom templates added to menu
       // (see KMMainWidget::updateCustomTemplateMenus)
       menu->addAction( mMsgActions->replyMenu() );
-      menu->addAction( mForwardActionMenu );
+      menu->addAction( mMsgActions->forwardMenu() );
       menu->addSeparator();
     }
 
@@ -452,16 +443,17 @@ void KMReaderMainWin::slotMsgPopup(KMMessage &aMsg, const KUrl &aUrl, const QPoi
     menu->addAction( mSaveAtmAction );
     menu->addAction( mMsgActions->createTodoAction() );
   }
-  menu->exec(aPoint, 0);
+  menu->exec( aPoint, 0 );
   delete menu;
 }
 
-void KMReaderMainWin::copySelectedToFolder( QAction* act )
+void KMReaderMainWin::slotCopySelectedMessagesToFolder( QAction* act )
 {
-  if (!mMenuToFolder[act])
+  KMFolder * f = static_cast<KMFolder *>( act->data().value<void *>() );
+  if ( !f )
     return;
 
-  KMCommand *command = new KMCopyCommand( mMenuToFolder[act], mMsg );
+  KMCommand *command = new KMCopyCommand( f, mMsg );
   command->start();
 }
 
