@@ -12,33 +12,32 @@
     Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "sievejob.h"
 
+#include <kjobtrackerinterface.h>
 #include <kio/job.h>
+#include <kio/deletejob.h>
+#include <kio/jobuidelegate.h>
 using KIO::Job;
 // <kio/global.h>
-using KIO::UDSAtomTypes;
 using KIO::UDSEntryList;
 using KIO::UDSEntry;
 #include <kdebug.h>
 
-#include <qtextcodec.h>
+#include <QTextCodec>
 
 #include <cassert>
 
 namespace KMail {
 
-  SieveJob::SieveJob( const KURL & url, const QString & script,
-		      const QValueStack<Command> & commands,
+  SieveJob::SieveJob( const KUrl & url, const QString & script,
+		      const QStack<Command> & commands,
 		      QObject * parent, const char * name )
-    : QObject( parent, name ),
+    : QObject( parent ),
       mUrl( url ), mJob( 0 ), mDec( 0 ),
       mScript( script ), mFileExists( DontKnow ), mCommands( commands )
   {
+    setObjectName( name );
     assert( !commands.isEmpty() );
     schedule( commands.top() );
   }
@@ -46,69 +45,73 @@ namespace KMail {
   SieveJob::~SieveJob() {
     kill();
     delete mDec;
-    kdDebug(5006) << "~SieveJob()" << endl;
+    kDebug() <<"~SieveJob()";
   }
 
-  void SieveJob::kill( bool quiet ) {
-    if ( mJob ) mJob->kill( quiet );
+  void SieveJob::kill( KJob::KillVerbosity verbosity ) {
+    if ( mJob ) mJob->kill( verbosity );
+  }
+
+  void SieveJob::setInteractive( bool interactive ) {
+    if ( mJob && !interactive ) {
+      mJob->setUiDelegate( 0 );
+      KIO::getJobTracker()->unregisterJob(mJob);
+    }
   }
 
   void SieveJob::schedule( Command command ) {
     switch ( command ) {
     case Get:
-      kdDebug(5006) << "SieveJob::schedule: get( " << mUrl.prettyURL() << " )" << endl;
+      kDebug() << "get(" << mUrl.prettyUrl() <<" )";
       mJob = KIO::get( mUrl );
       connect( mJob, SIGNAL(data(KIO::Job*,const QByteArray&)),
-	       SLOT(slotData(KIO::Job*,const QByteArray&)) );
+               SLOT(slotData(KIO::Job*,const QByteArray&)) );
       break;
     case Put:
-      kdDebug(5006) << "SieveJob::schedule: put( " << mUrl.prettyURL() << " )" << endl;
-      mJob = KIO::put( mUrl, 0600, true /*overwrite*/, false /*resume*/ );
+      kDebug() << "put(" << mUrl.prettyUrl() <<" )";
+      mJob = KIO::put( mUrl, 0600, KIO::Overwrite );
       connect( mJob, SIGNAL(dataReq(KIO::Job*,QByteArray&)),
-	       SLOT(slotDataReq(KIO::Job*,QByteArray&)) );
+               SLOT(slotDataReq(KIO::Job*,QByteArray&)) );
       break;
     case Activate:
-      kdDebug(5006) << "SieveJob::schedule: chmod( " << mUrl.prettyURL() << ", 0700 )"
-		<< endl;
+      kDebug() << "chmod(" << mUrl.prettyUrl() <<", 0700 )";
       mJob = KIO::chmod( mUrl, 0700 );
       break;
     case Deactivate:
-      kdDebug(5006) << "SieveJob::schedule: chmod( " << mUrl.prettyURL() << ", 0600 )"
-		<< endl;
+      kDebug() << "chmod(" << mUrl.prettyUrl() <<", 0600 )";
       mJob = KIO::chmod( mUrl, 0600 );
       break;
     case SearchActive:
-      kdDebug(5006) << "SieveJob::schedule: listDir( " << mUrl.prettyURL() << " )" << endl;
+      kDebug() << "listDir(" << mUrl.prettyUrl() <<" )";
       {
-	KURL url = mUrl;
-	QString query = url.query(); //save query part, because KURL::cd() erases it
-	if ( !url.fileName().isEmpty() )
-		url.cd("..");
-	url.setQuery( query );
-	kdDebug(5006) << "SieveJob::schedule: listDir's real URL: " << url.prettyURL()
-		  << endl;
-	mJob = KIO::listDir( url );
-	connect( mJob, SIGNAL(entries(KIO::Job*,const KIO::UDSEntryList&)),
-		 SLOT(slotEntries(KIO::Job*,const KIO::UDSEntryList&)) );
-	break;
+        KUrl url = mUrl;
+        QString query = url.query(); //save query part, because KUrl::cd() erases it
+        if ( !url.fileName().isEmpty() )
+          url.cd("..");
+        url.setQuery( query );
+        kDebug() << "listDir's real URL:" << url.prettyUrl();
+        mJob = KIO::listDir( url );
+        connect( mJob, SIGNAL(entries(KIO::Job*,const KIO::UDSEntryList&)),
+                 SLOT(slotEntries(KIO::Job*,const KIO::UDSEntryList&)) );
+        break;
       }
     case List:
-      kdDebug(5006) << "SieveJob::schedule: listDir( " << mUrl.prettyURL() << " )" << endl;
+      kDebug() << "listDir(" << mUrl.prettyUrl() <<" )";
       {
-	mJob = KIO::listDir( mUrl );
-	connect( mJob, SIGNAL( entries(KIO::Job *, const KIO::UDSEntryList & ) ),
-		 SLOT( slotEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
-	break;
+        mJob = KIO::listDir( mUrl );
+        connect( mJob, SIGNAL( entries(KIO::Job *, const KIO::UDSEntryList & ) ),
+                 SLOT( slotEntries( KIO::Job *, const KIO::UDSEntryList & ) ) );
+        break;
       }
     case Delete:
-      kdDebug(5006) << "SieveJob::schedule: delete( " << mUrl.prettyURL() << " )" << endl;
+      kDebug() << "delete(" << mUrl.prettyUrl() <<" )";
       mJob = KIO::del( mUrl );
       break;
     default:
       assert( 0 );
     }
     // common to all jobs:
-    connect( mJob, SIGNAL(result(KIO::Job*)), SLOT(slotResult(KIO::Job*)) );
+    connect( mJob, SIGNAL(result(KJob*)), SLOT(slotResult(KJob*)) );
   }
 
   void SieveJob::slotData( Job *, const QByteArray & data ) {
@@ -132,31 +135,25 @@ namespace KMail {
     }
 
     // Convert mScript into UTF-8:
-    data = mScript.utf8();
+    data = mScript.toUtf8();
 
     // "data" contains a trailing NUL, remove:
     if ( data.size() > 0 && data[(int)data.size() - 1] == '\0' )
       data.resize( data.size() - 1 );
 
     // mark mScript sent:
-    mScript = QString::null;
+    mScript.clear();
   }
 
   void SieveJob::slotEntries( Job *, const UDSEntryList & l ) {
     // loop over entries:
     for ( UDSEntryList::const_iterator it = l.begin() ; it != l.end() ; ++it ) {
-      // Loop over all UDS atoms to find the UDS_ACCESS and UDS_NAME atoms;
+      // Loop over all UDS atoms to find the UDSEntry::UDS_ACCESS and UDS_NAME atoms;
       // note if we find an exec'able file ( == active script )
       // or the requested filename (mUrl.fileName()).
-      QString filename;
-      bool isActive = false;
-      for ( UDSEntry::const_iterator et = (*it).begin() ; et != (*it).end() ; ++ et ) {
-	if ( ( *et ).m_uds == KIO::UDS_NAME ) {
-	  filename = ( *et ).m_str;
-	  mAvailableScripts.append( filename );
-	} else if ( ( *et ).m_uds == KIO::UDS_ACCESS && ( *et ).m_long == 0700 )
-	  isActive = true;
-      }
+      const QString filename = it->stringValue( KIO::UDSEntry::UDS_NAME );
+      mAvailableScripts.append( filename );
+      bool isActive = ( it->numberValue( KIO::UDSEntry::UDS_ACCESS ) == 0700 );
 
       if ( isActive )
 	mActiveScriptName = filename;
@@ -169,7 +166,7 @@ namespace KMail {
     }
   }
 
-  void SieveJob::slotResult( Job * job ) {
+  void SieveJob::slotResult( KJob * job ) {
     Command lastCmd = mCommands.top();
 
     // First, let's see if we come back from a SearchActive. If so, set
@@ -182,14 +179,16 @@ namespace KMail {
     delete mDec; mDec = 0;
 
     if ( mSieveCapabilities.empty() ) {
-      mSieveCapabilities = QStringList::split( ' ', job->queryMetaData( "sieveExtensions" ) );
-      kdDebug(5006) << "Received Sieve extensions supported:" << endl
-		    << mSieveCapabilities.join("\n") << endl;
+      mSieveCapabilities = static_cast<KIO::Job*>(job)->queryMetaData( "sieveExtensions" ).split(' ', QString::SkipEmptyParts );
+      kDebug() <<"Received Sieve extensions supported:\n" << mSieveCapabilities.join("\n");
     }
 
     // check for errors:
     if ( job->error() ) {
-      job->showErrorDialog( 0 );
+      if ( static_cast<KIO::Job*>(job)->ui() ) {
+        static_cast<KIO::Job*>(job)->ui()->setWindow( 0 );
+        static_cast<KIO::Job*>(job)->ui()->showErrorMessage();
+      }
 
       emit result( this, false, mScript, mUrl.fileName() == mActiveScriptName );
 
@@ -199,15 +198,15 @@ namespace KMail {
 	emit gotScript( this, false, mScript, mUrl.fileName() == mActiveScriptName );
 
       mJob = 0;
-      delete this;
+      deleteLater();
       return;
     }
 
     // check for new tasks:
     if ( !mCommands.empty() ) {
-      // Don't fail get'ting a non-existant script:
+      // Don't fail getting a non-existent script:
       if ( mCommands.top() == Get && mFileExists == No ) {
-	mScript = QString::null;
+	mScript.clear();
 	mCommands.pop();
       }
     }
@@ -221,7 +220,7 @@ namespace KMail {
 	emit gotScript( this, true, mScript, mUrl.fileName() == mActiveScriptName );
 
       mJob = 0; // deletes itself on returning from this slot
-      delete this;
+      deleteLater();
       return;
     } else {
       // schedule the next command:
@@ -229,9 +228,9 @@ namespace KMail {
     }
   }
 
-  SieveJob * SieveJob::put( const KURL & dest, const QString & script,
+  SieveJob * SieveJob::put( const KUrl & dest, const QString & script,
 			    bool makeActive, bool wasActive ) {
-    QValueStack<Command> commands;
+    QStack<Command> commands;
     if ( makeActive )
       commands.push( Activate );
     if ( wasActive )
@@ -240,28 +239,34 @@ namespace KMail {
     return new SieveJob( dest, script, commands );
   }
 
-  SieveJob * SieveJob::get( const KURL & src ) {
-    QValueStack<Command> commands;
+  SieveJob * SieveJob::get( const KUrl & src ) {
+    QStack<Command> commands;
     commands.push( Get );
     commands.push( SearchActive );
-    return new SieveJob( src, QString::null, commands );
+    return new SieveJob( src, QString(), commands );
   }
 
-  SieveJob * SieveJob::list( const KURL & src ) {
-    QValueStack<Command> commands;
+  SieveJob * SieveJob::list( const KUrl & src ) {
+    QStack<Command> commands;
     commands.push( List );
-    return new SieveJob( src, QString::null, commands );
+    return new SieveJob( src, QString(), commands );
   }
-  SieveJob * SieveJob::del( const KURL & url ) {
-    QValueStack<Command> commands;
+  SieveJob * SieveJob::del( const KUrl & url ) {
+    QStack<Command> commands;
     commands.push( Delete );
-    return new SieveJob( url, QString::null, commands );
+    return new SieveJob( url, QString(), commands );
   }
 
-  SieveJob * SieveJob::activate( const KURL & url ) {
-    QValueStack<Command> commands;
+  SieveJob * SieveJob::desactivate( const KUrl & url ) {
+    QStack<Command> commands;
+    commands.push( Deactivate );
+    return new SieveJob( url, QString(), commands );
+  }
+
+  SieveJob * SieveJob::activate( const KUrl & url ) {
+    QStack<Command> commands;
     commands.push( Activate );
-    return new SieveJob( url, QString::null, commands );
+    return new SieveJob( url, QString(), commands );
   }
 
 } // namespace KMail
