@@ -36,7 +36,6 @@
 
 #include <kglobalsettings.h>
 
-#include <Q3Frame>
 #include <QAbstractEventDispatcher>
 #include <QApplication>
 #include <QToolTip>
@@ -47,8 +46,8 @@
 #include <QResizeEvent>
 #include <QVBoxLayout>
 
-KNoteTip::KNoteTip( K3IconView *parent )
-  : Q3Frame( 0, 0, Qt::WX11BypassWM |   // this will make Seli happy >:-P
+KNoteTip::KNoteTip( QListWidget *parent )
+  : QFrame( 0, Qt::WX11BypassWM |   // this will make Seli happy >:-P
              Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool | Qt::WStyle_StaysOnTop ),
     mFilter( false ),
     mView( parent ),
@@ -61,10 +60,9 @@ KNoteTip::KNoteTip( K3IconView *parent )
 
   QBoxLayout *layout = new QVBoxLayout( this );
   layout->addWidget( mPreview );
-
+  layout->setMargin( 1 );
   setPalette( QToolTip::palette() );
-  setMargin( 1 );
-  setFrameStyle( Q3Frame::Plain | Q3Frame::Box );
+  setFrameStyle( QFrame::Plain );
   hide();
 }
 
@@ -81,7 +79,6 @@ void KNoteTip::setNote( KNotesIconViewItem *item )
   }
 
   mNoteIVI = item;
-
   if ( !mNoteIVI ) {
     QAbstractEventDispatcher::instance()->unregisterTimers(this);
     if ( isVisible() ) {
@@ -106,7 +103,8 @@ void KNoteTip::setNote( KNotesIconViewItem *item )
     while ( w > 60 && h == mPreview->heightForWidth( w - 20 ) ) {
       w -= 20;
     }
-    QRect desk = KGlobalSettings::desktopGeometry( mNoteIVI->rect().center() );
+
+    QRect desk = KGlobalSettings::desktopGeometry( mView->visualItemRect(mNoteIVI ).center() );
     resize( w, qMin( h, desk.height() / 2 - 20 ) );
 
     hide();
@@ -120,7 +118,7 @@ void KNoteTip::setNote( KNotesIconViewItem *item )
 
 void KNoteTip::resizeEvent( QResizeEvent *ev )
 {
-  Q3Frame::resizeEvent( ev );
+  QFrame::resizeEvent( ev );
   reposition();
 }
 
@@ -204,12 +202,11 @@ void KNoteTip::reposition()
     return;
   }
 
-  QRect rect = mNoteIVI->rect();
-  QPoint off = mView->mapToGlobal( mView->contentsToViewport( QPoint( 0, 0 ) ) );
+  QRect rect = mView->visualItemRect( mNoteIVI);
+  QPoint off = mView->mapFromParent( mView->viewport()->mapToGlobal( QPoint( 0,0 ) ) );
   rect.translate( off.x(), off.y() );
 
   QPoint pos = rect.center();
-
   // should the tooltip be shown to the left or to the right of the ivi?
   QRect desk = KGlobalSettings::desktopGeometry( pos );
   if ( rect.center().x() + width() > desk.right() ) {
@@ -228,7 +225,6 @@ void KNoteTip::reposition()
   } else {
     pos.setY( rect.bottom() );
   }
-
   move( pos );
   update();
 }
