@@ -24,56 +24,44 @@
 */
 
 #include "koprefsdialog.h"
-#include "koprefs.h"
-#include "ktimeedit.h"
+#include "kocore.h"
 #include "koglobals.h"
+#include "koprefs.h"
 #include "stdcalendar.h"
-#include "calendar/calendardecoration.h"
-#include "korganizer/printplugin.h"
+#include "ui_kogroupwareprefspage.h"
 
-#include <kholidays.h>
-using namespace LibKHolidays;
+#include <libkdepim/ktimeedit.h>
 
-#include <kcal/calendarresources.h>
+#include <KCal/CalendarResources>
 
-#include <kvbox.h>
-#include <kcolorbutton.h>
-#include <klocale.h>
-#include <kglobal.h>
-#include <kmessagebox.h>
-#include <kiconloader.h>
-#include <kemailsettings.h>
-#include <kcalendarsystem.h>
-#include <ksystemtimezone.h>
-#include <kpushbutton.h>
-#include <kocore.h>
-#include <kstandarddirs.h>
-#include <kconfig.h>
-#include <KLineEdit>
+#include <KHolidays/Holidays>
+using namespace KHolidays;
+
+#include <KCalendarSystem>
+#include <KColorButton>
 #include <KComboBox>
+#include <KHBox>
+#include <KIntSpinBox>
+#include <KLocale>
+#include <KMessageBox>
+#include <KPushButton>
+#include <KService>
+#include <KStandardDirs>
+#include <KSystemTimeZones>
 #include <KTabWidget>
 
-#include <q3listview.h>
-#include <q3buttongroup.h>
-#include <QLayout>
-#include <QFormLayout>
-#include <QLabel>
-#include <QSlider>
-#include <QFile>
-#include <QSpinBox>
-#include <QCheckBox>
-#include <QRadioButton>
-#include <QPushButton>
-#include <QTextStream>
-#include <QGridLayout>
+#include <Q3ButtonGroup>
+#include <Q3ListView>
 #include <QBoxLayout>
-#include <QHBoxLayout>
-#include <QVBoxLayout>
+#include <QFormLayout>
+#include <QGridLayout>
 #include <QGroupBox>
+#include <QLabel>
+#include <QPushButton>
+#include <QRadioButton>
 #include <QTimeEdit>
 #include <QTreeWidget>
-
-#include "ui_kogroupwareprefspage.h"
+#include <QVBoxLayout>
 
 KOPrefsDialogMain::KOPrefsDialogMain( const KComponentData &inst, QWidget *parent )
   : KPrefsModule( KOPrefs::instance(), inst, parent )
@@ -83,7 +71,7 @@ KOPrefsDialogMain::KOPrefsDialogMain( const KComponentData &inst, QWidget *paren
   topTopLayout->addWidget( tabWidget );
 
   // Personal Settings
-  QFrame *personalFrame = new QFrame( this );
+  QWidget *personalFrame = new QWidget( this );
   QVBoxLayout *personalLayout = new QVBoxLayout( personalFrame );
   tabWidget->addTab( personalFrame, KIcon( "preferences-desktop-personal" ),
                      i18nc( "@title:tab personal settings", "Personal" ) );
@@ -220,8 +208,14 @@ class KOPrefsDialogTime : public KPrefsModule
       QGridLayout *regionalLayout = new QGridLayout( regionalPage );
       regionalLayout->setSpacing( KDialog::spacingHint() );
 
+      QGroupBox *datetimeGroupBox =
+        new QGroupBox( i18nc( "@title:group", "General Time and Date" ), regionalPage );
+      regionalLayout->addWidget( datetimeGroupBox, 0, 0 );
+
+      QGridLayout *datetimeLayout = new QGridLayout( datetimeGroupBox );
+
       KHBox *timeZoneBox = new KHBox( regionalPage );
-      regionalLayout->addWidget( timeZoneBox, 0, 0, 1, 2 );
+      datetimeLayout->addWidget( timeZoneBox, 0, 0, 1, 2 );
 
       QLabel *timeZoneLabel = new QLabel( i18nc( "@label", "Time zone:" ), timeZoneBox );
       QString whatsThis = i18nc( "@info:whatsthis",
@@ -265,9 +259,20 @@ class KOPrefsDialogTime : public KPrefsModule
       mTimeZoneCombo->setCurrentIndex( nCurrentlySet );
       mTimeZoneCombo->setWhatsThis( whatsThis );
 
+      KPrefsWidTime *dayBegins =
+        addWidTime( KOPrefs::instance()->dayBeginsItem(), regionalPage );
+      datetimeLayout->addWidget( dayBegins->label(), 1, 0 );
+      datetimeLayout->addWidget( dayBegins->timeEdit(), 1, 1 );
+
+      QGroupBox *holidaysGroupBox =
+        new QGroupBox( i18nc( "@title:group", "Holidays" ), regionalPage );
+      regionalLayout->addWidget( holidaysGroupBox, 1, 0 );
+
+      QGridLayout *holidaysLayout = new QGridLayout( holidaysGroupBox );
+
       // holiday region selection
       KHBox *holidayRegBox = new KHBox( regionalPage );
-      regionalLayout->addWidget( holidayRegBox, 1, 0, 1, 2 );
+      holidaysLayout->addWidget( holidayRegBox, 1, 0, 1, 2 );
 
       QLabel *holidayLabel = new QLabel(
         i18nc( "@label", "Use holiday region:" ), holidayRegBox );
@@ -280,7 +285,7 @@ class KOPrefsDialogTime : public KPrefsModule
 
       QString currentHolidayName;
       QStringList holidayList;
-      QStringList countryList = KHolidays::locations();
+      QStringList countryList = HolidayRegion::locations();
 
       foreach ( const QString &country, countryList ) {
         QString countryFile = KStandardDirs::locate( "locale",
@@ -316,14 +321,9 @@ class KOPrefsDialogTime : public KPrefsModule
         }
       }
 
-      KPrefsWidTime *dayBegins =
-        addWidTime( KOPrefs::instance()->dayBeginsItem(), regionalPage );
-      regionalLayout->addWidget( dayBegins->label(), 2, 0 );
-      regionalLayout->addWidget( dayBegins->timeEdit(), 2, 1 );
-
       QGroupBox *workingHoursGroupBox =
         new QGroupBox( i18nc( "@title:group", "Working Hours" ), regionalPage );
-      regionalLayout->addWidget( workingHoursGroupBox, 3, 0, 1, 2 );
+      regionalLayout->addWidget( workingHoursGroupBox, 2, 0 );
 
       QBoxLayout *workingHoursLayout = new QVBoxLayout( workingHoursGroupBox );
 
@@ -385,30 +385,52 @@ class KOPrefsDialogTime : public KPrefsModule
       QGridLayout *defaultLayout = new QGridLayout( defaultPage );
       defaultLayout->setSpacing( KDialog::spacingHint() );
 
+      QGroupBox *timesGroupBox =
+        new QGroupBox( i18nc( "@title:group", "Appointments" ), defaultPage );
+      defaultLayout->addWidget( timesGroupBox, 0, 0 );
+
+      QGridLayout *timesLayout = new QGridLayout( timesGroupBox );
+
       KPrefsWidTime *defaultTime =
         addWidTime( KOPrefs::instance()->startTimeItem(), defaultPage );
-      defaultLayout->addWidget( defaultTime->label(), 0, 0 );
-      defaultLayout->addWidget( defaultTime->timeEdit(), 0, 1 );
+      timesLayout->addWidget( defaultTime->label(), 0, 0 );
+      timesLayout->addWidget( defaultTime->timeEdit(), 0, 1 );
 
       KPrefsWidDuration *defaultDuration =
-        addWidDuration( KOPrefs::instance()->defaultDurationItem(), defaultPage );
-      defaultLayout->addWidget( defaultDuration->label(), 1, 0 );
-      defaultLayout->addWidget( defaultDuration->timeEdit(), 1, 1 );
+        addWidDuration( KOPrefs::instance()->defaultDurationItem(), "hh:mm", defaultPage );
 
-      QStringList alarmList;
-      alarmList << i18nc( "@item:inlistbox", "1 minute" )
-                << i18nc( "@item:inlistbox", "5 minutes" )
-                << i18nc( "@item:inlistbox", "10 minutes" )
-                << i18nc( "@item:inlistbox", "15 minutes" )
-                << i18nc( "@item:inlistbox", "30 minutes" );
-      QLabel *alarmLabel = new QLabel( i18nc( "@label", "Default reminder time:" ), defaultPage );
-      defaultLayout->addWidget( alarmLabel, 2, 0 );
-      alarmLabel->setWhatsThis( KOPrefs::instance()->alarmTimeItem()->whatsThis() );
-      mAlarmTimeCombo = new KComboBox( defaultPage );
-      mAlarmTimeCombo->setWhatsThis( KOPrefs::instance()->alarmTimeItem()->whatsThis() );
-      connect( mAlarmTimeCombo, SIGNAL(activated(int)), SLOT(slotWidChanged()) );
-      mAlarmTimeCombo->addItems( alarmList );
-      defaultLayout->addWidget( mAlarmTimeCombo, 2, 1 );
+      timesLayout->addWidget( defaultDuration->label(), 1, 0 );
+      timesLayout->addWidget( defaultDuration->timeEdit(), 1, 1 );
+
+      QGroupBox *remindersGroupBox =
+        new QGroupBox( i18nc( "@title:group", "Reminders" ), defaultPage );
+      defaultLayout->addWidget( remindersGroupBox, 1, 0 );
+
+      QGridLayout *remindersLayout = new QGridLayout( remindersGroupBox );
+
+      QLabel *reminderLabel =
+        new QLabel( i18nc( "@label", "Default reminder time:" ), defaultPage );
+      remindersLayout->addWidget( reminderLabel, 0, 0 );
+      reminderLabel->setWhatsThis( KOPrefs::instance()->reminderTimeItem()->whatsThis() );
+      mReminderTimeSpin  = new KIntSpinBox( defaultPage );
+      mReminderTimeSpin->setWhatsThis( KOPrefs::instance()->reminderTimeItem()->whatsThis() );
+      mReminderTimeSpin->setToolTip( KOPrefs::instance()->reminderTimeItem()->toolTip() );
+      connect( mReminderTimeSpin, SIGNAL(valueChanged(int)), SLOT(slotWidChanged()) );
+      remindersLayout->addWidget( mReminderTimeSpin, 0, 1 );
+
+      mReminderUnitsCombo = new KComboBox( defaultPage );
+      mReminderUnitsCombo->setToolTip(
+        KOPrefs::instance()->reminderTimeUnitsItem()->toolTip() );
+      mReminderUnitsCombo->setWhatsThis(
+        KOPrefs::instance()->reminderTimeUnitsItem()->whatsThis() );
+      connect( mReminderUnitsCombo, SIGNAL(activated(int)), SLOT(slotWidChanged()) );
+      mReminderUnitsCombo->addItem(
+        i18nc( "@item:inlistbox reminder units in minutes", "minute(s)" ) );
+      mReminderUnitsCombo->addItem(
+        i18nc( "@item:inlistbox reminder time units in hours", "hour(s)" ) );
+      mReminderUnitsCombo->addItem(
+        i18nc( "@item:inlistbox reminder time units in days", "day(s)" ) );
+      remindersLayout->addWidget( mReminderUnitsCombo, 0, 2 );
 
       defaultLayout->setRowStretch( 3, 1 );
       load();
@@ -422,7 +444,8 @@ class KOPrefsDialogTime : public KPrefsModule
         setCombo( mTimeZoneCombo, i18n( tz.name().toUtf8() ) );
       }
 
-      mAlarmTimeCombo->setCurrentIndex( KOPrefs::instance()->mAlarmTime );
+      mReminderTimeSpin->setValue( KOPrefs::instance()->mReminderTime );
+      mReminderUnitsCombo->setCurrentIndex( KOPrefs::instance()->mReminderTimeUnits );
       for ( int i = 0; i < 7; ++i ) {
         mWorkDays[i]->setChecked( ( 1 << i ) & ( KOPrefs::instance()->mWorkWeekMask ) );
       }
@@ -448,7 +471,8 @@ class KOPrefsDialogTime : public KPrefsModule
                                        QString() :
                                        mRegionMap[mHolidayCombo->currentText()];
 
-      KOPrefs::instance()->mAlarmTime = mAlarmTimeCombo->currentIndex();
+      KOPrefs::instance()->mReminderTime = mReminderTimeSpin->value();
+      KOPrefs::instance()->mReminderTimeUnits = mReminderUnitsCombo->currentIndex();
       int mask = 0;
       for ( int i = 0; i < 7; ++i ) {
         if ( mWorkDays[i]->isChecked() ) {
@@ -481,7 +505,8 @@ class KOPrefsDialogTime : public KPrefsModule
     QStringList   tzonenames;
     KComboBox    *mHolidayCombo;
     QMap<QString,QString> mRegionMap;
-    KComboBox    *mAlarmTimeCombo;
+    KIntSpinBox  *mReminderTimeSpin;
+    KComboBox    *mReminderUnitsCombo;
     QCheckBox    *mWorkDays[7];
 };
 
@@ -507,55 +532,63 @@ class KOPrefsDialogViews : public KPrefsModule
       KTabWidget *tabWidget = new KTabWidget( this );
       topTopLayout->addWidget( tabWidget );
 
-      QFrame *topFrame = new QFrame( this );
-      tabWidget->addTab( topFrame, KIcon( "view-choose" ),
+      // Tab: Views->General
+      QFrame *generalFrame = new QFrame( this );
+      tabWidget->addTab( generalFrame, KIcon( "view-choose" ),
                          i18nc( "@title:tab general settings", "General" ) );
-      QBoxLayout *generalLayout = new QVBoxLayout( topFrame );
-      generalLayout->setSpacing( KDialog::spacingHint() );
 
-      KPrefsWidBool *enableToolTips =
-        addWidBool( KOPrefs::instance()->enableToolTipsItem(), topFrame );
-      generalLayout->addWidget( enableToolTips->checkBox() );
+      QBoxLayout *generalLayout = new QVBoxLayout( generalFrame );
 
-      KPrefsWidBool *showTodosAgenda =
-        addWidBool( KOPrefs::instance()->showAllDayTodoItem(), topFrame );
-      generalLayout->addWidget( showTodosAgenda->checkBox() );
-
-      KPrefsWidBool *todosUseCategoryColors =
-        addWidBool( KOPrefs::instance()->todosUseCategoryColorsItem(), topFrame );
-      generalLayout->addWidget( todosUseCategoryColors->checkBox() );
-
-      QGroupBox *dateNavGroup =
-        new QGroupBox( i18nc( "@title:group", "Date Navigator" ), topFrame );
-      generalLayout->addWidget( dateNavGroup );
-
-      QBoxLayout *dateNavLayout = new QVBoxLayout( dateNavGroup );
-
-      dateNavLayout->addWidget( addWidBool( KOPrefs::instance()->dailyRecurItem() )->checkBox() );
-      dateNavLayout->addWidget( addWidBool( KOPrefs::instance()->weeklyRecurItem() )->checkBox() );
+      // GroupBox: Views->General->Display Options
+      QVBoxLayout *gdisplayLayout = new QVBoxLayout;
+      QGroupBox *gdisplayBox = new QGroupBox( i18nc( "@title:group", "Display Options" ) );
+      gdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->enableToolTipsItem() )->checkBox() );
+      gdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->todosUseCategoryColorsItem() )->checkBox() );
+      gdisplayBox->setLayout( gdisplayLayout );
+      generalLayout->addWidget( gdisplayBox );
 
+      // GroupBox: Views->General->Date Navigator
+      QVBoxLayout *datenavLayout = new QVBoxLayout;
+      QGroupBox *datenavBox = new QGroupBox( i18nc( "@title:group", "Date Navigator" ) );
+      datenavLayout->addWidget(
+        addWidBool( KOPrefs::instance()->dailyRecurItem() )->checkBox() );
+      datenavLayout->addWidget(
+        addWidBool( KOPrefs::instance()->weeklyRecurItem() )->checkBox() );
+      datenavLayout->addWidget(
+        addWidBool( KOPrefs::instance()->highlightTodosItem() )->checkBox() );
+      datenavLayout->addWidget(
+        addWidBool( KOPrefs::instance()->highlightJournalsItem() )->checkBox() );
+      datenavBox->setLayout( datenavLayout );
+      generalLayout->addWidget( datenavBox );
       generalLayout->addStretch( 1 );
 
+      // Tab: Views->Agenda View
       QFrame *agendaFrame = new QFrame( this );
       tabWidget->addTab( agendaFrame, KIcon( "view-calendar-workweek" ),
                          i18nc( "@title:tab", "Agenda View" ) );
 
       QBoxLayout *agendaLayout = new QVBoxLayout( agendaFrame );
 
-      QBoxLayout *hourSizeLayout = new QHBoxLayout;
-      agendaLayout->addLayout( hourSizeLayout );
+      // GroupBox: Views->Agenda View->Display Options
+      QVBoxLayout *adisplayLayout = new QVBoxLayout;
+      QGroupBox *adisplayBox = new QGroupBox( i18nc( "@title:group", "Display Options" ) );
+
+      QHBoxLayout *hourSizeLayout = new QHBoxLayout;
+      adisplayLayout->addLayout( hourSizeLayout );
 
       KPrefsWidInt *hourSize =
         addWidInt( KOPrefs::instance()->hourSizeItem() );
       hourSize->spinBox()->setSuffix(
-        i18nc( "@label suffix in the hour size spin box", " pixel" ) );
+        i18nc( "@label suffix in the hour size spin box", " pixels" ) );
 
       hourSizeLayout->addWidget( hourSize->label() );
       hourSizeLayout->addWidget( hourSize->spinBox() );
       hourSizeLayout->addStretch( 1 );
 
       QBoxLayout *nextDaysLayout = new QHBoxLayout;
-      agendaLayout->addLayout( nextDaysLayout );
+      adisplayLayout->addLayout( nextDaysLayout );
 
       KPrefsWidInt *nextDays =
         addWidInt( KOPrefs::instance()->nextXDaysItem() );
@@ -566,55 +599,88 @@ class KOPrefsDialogViews : public KPrefsModule
       nextDaysLayout->addWidget( nextDays->spinBox() );
       nextDaysLayout->addStretch( 1 );
 
+      adisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->enableAgendaItemIconsItem() )->checkBox() );
+      adisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->showTodosAgendaViewItem() )->checkBox() );
       KPrefsWidBool *marcusBainsEnabled =
         addWidBool( KOPrefs::instance()->marcusBainsEnabledItem() );
-      agendaLayout->addWidget( marcusBainsEnabled->checkBox() );
+      adisplayLayout->addWidget( marcusBainsEnabled->checkBox() );
 
       KPrefsWidBool *marcusBainsShowSeconds =
         addWidBool( KOPrefs::instance()->marcusBainsShowSecondsItem() );
       connect( marcusBainsEnabled->checkBox(), SIGNAL(toggled(bool)),
                marcusBainsShowSeconds->checkBox(), SLOT(setEnabled(bool)) );
 
-      agendaLayout->addWidget( marcusBainsShowSeconds->checkBox() );
-      agendaLayout->addWidget(
+      adisplayLayout->addWidget( marcusBainsShowSeconds->checkBox() );
+      adisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->selectionStartsEditorItem() )->checkBox() );
+
+      adisplayBox->setLayout( adisplayLayout );
+      agendaLayout->addWidget( adisplayBox );
+
+      // GroupBox: Views->Agenda View->Color Usage
       agendaLayout->addWidget(
-        addWidCombo( KOPrefs::instance()->agendaViewColorsItem() )->comboBox() );
+        addWidRadios( KOPrefs::instance()->agendaViewColorsItem() )->groupBox() );
+
+      // GroupBox: Views->Agenda View->Multiple Calendars
       agendaLayout->addWidget(
-        addWidCombo( KOPrefs::instance()->agendaViewCalendarDisplayItem() )->comboBox() );
+        addWidRadios( KOPrefs::instance()->agendaViewCalendarDisplayItem() )->groupBox() );
 
       agendaLayout->addStretch( 1 );
 
+      // Tab: Views->Month View
       QFrame *monthFrame = new QFrame( this );
       tabWidget->addTab( monthFrame, KIcon( "view-calendar-month" ),
                          i18nc( "@title:tab", "Month View" ) );
 
       QBoxLayout *monthLayout = new QVBoxLayout( monthFrame );
 
-      monthLayout->addWidget(
+      // GroupBox: Views->Month View->Display Options
+      QVBoxLayout *mdisplayLayout = new QVBoxLayout;
+      QGroupBox *mdisplayBox = new QGroupBox( i18nc( "@title:group", "Display Options" ) );
+      mdisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->enableMonthScrollItem() )->checkBox() );
-      monthLayout->addWidget(
+      mdisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->enableMonthItemIconsItem() )->checkBox() );
-      monthLayout->addWidget(
+      mdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->showTodosMonthViewItem() )->checkBox() );
+      mdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->showJournalsMonthViewItem() )->checkBox() );
+      mdisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->fullViewMonthItem() )->checkBox() );
-      monthLayout->addWidget(
-        addWidBool( KOPrefs::instance()->monthViewUsesCategoryColorItem() )->checkBox() );
-      monthLayout->addWidget(
-        addWidBool( KOPrefs::instance()->monthViewUsesResourceColorItem() )->checkBox() );
+      mdisplayBox->setLayout( mdisplayLayout );
+      monthLayout->addWidget( mdisplayBox );
 
+      // GroupBox: Views->Month View->Color Usage
+      monthLayout->addWidget(
+        addWidRadios( KOPrefs::instance()->monthViewColorsItem() )->groupBox() );
       monthLayout->addStretch( 1 );
 
+      // Tab: Views->Todo View
       QFrame *todoFrame = new QFrame( this );
       tabWidget->addTab( todoFrame, KIcon( "view-calendar-tasks" ),
                          i18nc( "@title:tab", "Todo View" ) );
 
       QBoxLayout *todoLayout = new QVBoxLayout( todoFrame );
-      todoLayout->addWidget(
-        addWidBool( KOPrefs::instance()->fullViewTodoItem() )->checkBox() );
-      todoLayout->addWidget(
-        addWidBool( KOPrefs::instance()->recordTodosInJournalsItem() )->checkBox() );
-      todoLayout->addWidget(
+
+      // GroupBox: Views->Todo View->Display Options
+      QVBoxLayout *tdisplayLayout = new QVBoxLayout;
+      QGroupBox *tdisplayBox = new QGroupBox( i18nc( "@title:group", "Display Options" ) );
+      tdisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->sortCompletedTodosSeparatelyItem() )->checkBox() );
+      tdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->fullViewTodoItem() )->checkBox() );
+      tdisplayBox->setLayout( tdisplayLayout );
+      todoLayout->addWidget( tdisplayBox );
+
+      // GroupBox: Views->Todo View->Other
+      QVBoxLayout *otherLayout = new QVBoxLayout;
+      QGroupBox *otherBox = new QGroupBox( i18nc( "@title:group", "Other Options" ) );
+      otherLayout->addWidget(
+        addWidBool( KOPrefs::instance()->recordTodosInJournalsItem() )->checkBox() );
+      otherBox->setLayout( otherLayout );
+      todoLayout->addWidget( otherBox );
       todoLayout->addStretch( 1 );
 
       load();
@@ -653,45 +719,53 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
   colorLayout->addWidget( holidayColor->label(), 0, 0 );
   colorLayout->addWidget( holidayColor->button(), 0, 1 );
 
-  KPrefsWidColor *eventColor =
-    addWidColor( KOPrefs::instance()->agendaCalendarItemsEventsBackgroundColorItem(), colorFrame );
-  colorLayout->addWidget( eventColor->label(), 2, 0 );
-  colorLayout->addWidget( eventColor->button(), 2, 1 );
-
   // agenda view background color
   KPrefsWidColor *agendaBgColor =
     addWidColor( KOPrefs::instance()->agendaGridBackgroundColorItem(), colorFrame );
   colorLayout->addWidget( agendaBgColor->label(), 3, 0 );
   colorLayout->addWidget( agendaBgColor->button(), 3, 1 );
 
+  // agenda view Marcus Bains line color
+  KPrefsWidColor *mblColor =
+    addWidColor( KOPrefs::instance()->agendaMarcusBainsLineLineColorItem(), colorFrame );
+  colorLayout->addWidget( mblColor->label(), 4, 0 );
+  colorLayout->addWidget( mblColor->button(), 4, 1 );
+
   // working hours color
   KPrefsWidColor *agendaGridWorkHoursBackgroundColor =
     addWidColor( KOPrefs::instance()->agendaGridWorkHoursBackgroundColorItem(), colorFrame );
-  colorLayout->addWidget( agendaGridWorkHoursBackgroundColor->label(), 4, 0 );
-  colorLayout->addWidget( agendaGridWorkHoursBackgroundColor->button(), 4, 1 );
+  colorLayout->addWidget( agendaGridWorkHoursBackgroundColor->label(), 5, 0 );
+  colorLayout->addWidget( agendaGridWorkHoursBackgroundColor->button(), 5, 1 );
 
   // Todo due today color
   KPrefsWidColor *todoDueTodayColor =
     addWidColor(
       KOPrefs::instance()->agendaCalendarItemsToDosDueTodayBackgroundColorItem(), colorFrame );
-  colorLayout->addWidget( todoDueTodayColor->label(), 5, 0 );
-  colorLayout->addWidget( todoDueTodayColor->button(), 5, 1 );
+  colorLayout->addWidget( todoDueTodayColor->label(), 6, 0 );
+  colorLayout->addWidget( todoDueTodayColor->button(), 6, 1 );
 
   // Todo overdue color
   KPrefsWidColor *todoOverdueColor =
     addWidColor(
       KOPrefs::instance()->agendaCalendarItemsToDosOverdueBackgroundColorItem(), colorFrame );
-  colorLayout->addWidget( todoOverdueColor->label(), 6, 0 );
-  colorLayout->addWidget( todoOverdueColor->button(), 6, 1 );
+  colorLayout->addWidget( todoOverdueColor->label(), 7, 0 );
+  colorLayout->addWidget( todoOverdueColor->button(), 7, 1 );
 
   // categories colors
   QGroupBox *categoryGroup = new QGroupBox( i18nc( "@title:group", "Categories" ), colorFrame );
-  colorLayout->addWidget( categoryGroup, 7, 0, 1, 2 );
+  colorLayout->addWidget( categoryGroup, 8, 0, 1, 2 );
 
-  QBoxLayout *categoryLayout = new QHBoxLayout;
+  QGridLayout *categoryLayout = new QGridLayout;
   categoryGroup->setLayout( categoryLayout );
 
-  mCategoryCombo = new KComboBox(categoryGroup);
+  KPrefsWidColor *unsetCategoryColor =
+    addWidColor( KOPrefs::instance()->unsetCategoryColorItem(), categoryGroup );
+  categoryLayout->addWidget( unsetCategoryColor->label(), 0, 0 );
+  categoryLayout->addWidget( unsetCategoryColor->button(), 0, 1 );
+  unsetCategoryColor->label()->setWhatsThis( unsetCategoryColor->button()->whatsThis() );
+  unsetCategoryColor->label()->setToolTip( unsetCategoryColor->button()->toolTip() );
+
+  mCategoryCombo = new KComboBox( categoryGroup );
   mCategoryCombo->addItems( KOPrefs::instance()->mCustomCategories );
   mCategoryCombo->setWhatsThis(
     i18nc( "@info:whatsthis",
@@ -699,7 +773,7 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
            "You can change the selected category color using "
            "the button below." ) );
   connect( mCategoryCombo, SIGNAL(activated(int)), SLOT(updateCategoryColor()) );
-  categoryLayout->addWidget( mCategoryCombo );
+  categoryLayout->addWidget( mCategoryCombo, 1, 0 );
 
   mCategoryButton = new KColorButton( categoryGroup );
   mCategoryButton->setWhatsThis(
@@ -707,18 +781,18 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
            "Choose here the color of the event category selected "
            "using the combo box above." ) );
   connect( mCategoryButton, SIGNAL(changed(const QColor &)), SLOT(setCategoryColor()) );
-  categoryLayout->addWidget( mCategoryButton );
+  categoryLayout->addWidget( mCategoryButton, 1, 1 );
 
   updateCategoryColor();
 
   // resources colors
   QGroupBox *resourceGroup = new QGroupBox( i18nc( "@title:group", "Resources" ), colorFrame );
-  colorLayout->addWidget( resourceGroup, 8, 0, 1, 2 );
+  colorLayout->addWidget( resourceGroup, 9, 0, 1, 2 );
 
   QBoxLayout *resourceLayout = new QHBoxLayout;
   resourceGroup->setLayout( resourceLayout );
 
-  mResourceCombo = new KComboBox(resourceGroup);
+  mResourceCombo = new KComboBox( resourceGroup );
   mResourceCombo->setWhatsThis(
     i18nc( "@info:whatsthis",
            "Select here resource you want to modify. "
@@ -737,7 +811,7 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
 
   updateResources();
 
-  colorLayout->setRowStretch( 9, 1 );
+  colorLayout->setRowStretch( 10, 1 );
 
   QWidget *fontFrame = new QWidget( this );
   tabWidget->addTab( fontFrame, KIcon( "preferences-desktop-font" ),
@@ -864,9 +938,9 @@ void KOPrefsDialogColorsAndFonts::setResourceColor()
 
 void KOPrefsDialogColorsAndFonts::updateResourceColor()
 {
-  QString res= mResourceIdentifier[mResourceCombo->currentIndex()];
+  QString res = mResourceIdentifier[mResourceCombo->currentIndex()];
   QColor color = mCategoryDict.value( res );
-  if ( color.isValid() )  {
+  if ( !color.isValid() )  {
     color = KOPrefs::instance()->resourceColor( res );
   }
   if ( color.isValid() ) {
@@ -911,16 +985,6 @@ KOPrefsDialogGroupScheduling::KOPrefsDialogGroupScheduling( const KComponentData
     addWidRadios( KOPrefs::instance()->mailClientItem(), topFrame );
   topLayout->addWidget( mailClientGroup->groupBox(), 2, 0, 1, 2 );
 
-#if 0
-  KPrefsWidRadios *schedulerGroup =
-    addWidRadios(
-      i18nc( "@title", "Scheduler Mail Client" ), KOPrefs::instance()->mIMIPScheduler, topFrame );
-  schedulerGroup->addRadio( "Dummy" ); // Only for debugging
-  schedulerGroup->addRadio( i18nc( "@option:radio", "Mail client" ) );
-
-  topLayout->addWidget( schedulerGroup->groupBox(), 0, 0, 1, 2 );
-#endif
-
   QLabel *aMailsLabel = new QLabel(
     i18nc( "@label", "Additional email addresses:" ), topFrame );
   QString whatsThis = i18nc( "@info:whatsthis",
@@ -1255,14 +1319,14 @@ KOPrefsDialogPlugins::KOPrefsDialogPlugins( const KComponentData &inst, QWidget
   topLayout->addWidget( buttonRow );
 
   mPositioningGroupBox = new QGroupBox( i18nc( "@title:group", "Position" ), topFrame );
-  mPositionMonthTop = new QCheckBox(
-    i18nc( "@option:check", "Show in the month view" ), mPositioningGroupBox );
+  //mPositionMonthTop = new QCheckBox(
+  //i18nc( "@option:check", "Show in the month view" ), mPositioningGroupBox );
   mPositionAgendaTop = new QRadioButton(
     i18nc( "@option:check", "Show at the top of the agenda views" ), mPositioningGroupBox );
   mPositionAgendaBottom = new QRadioButton(
     i18nc( "@option:check", "Show at the bottom of the agenda views" ), mPositioningGroupBox );
   QVBoxLayout *positioningLayout = new QVBoxLayout( mPositioningGroupBox );
-  positioningLayout->addWidget( mPositionMonthTop );
+  //positioningLayout->addWidget( mPositionMonthTop );
   positioningLayout->addWidget( mPositionAgendaTop );
   positioningLayout->addWidget( mPositionAgendaBottom );
   positioningLayout->addStretch( 1 );
@@ -1270,7 +1334,7 @@ KOPrefsDialogPlugins::KOPrefsDialogPlugins( const KComponentData &inst, QWidget
 
   connect( mConfigureButton, SIGNAL(clicked()), SLOT(configure()) );
 
-  connect( mPositionMonthTop, SIGNAL(clicked()), SLOT(positioningChanged()) );
+  //connect( mPositionMonthTop, SIGNAL(clicked()), SLOT(positioningChanged()) );
   connect( mPositionAgendaTop, SIGNAL(clicked()), SLOT(positioningChanged()) );
   connect( mPositionAgendaBottom, SIGNAL(clicked()), SLOT(positioningChanged()) );
 
@@ -1294,20 +1358,24 @@ void KOPrefsDialogPlugins::usrReadConfig()
   QTreeWidgetItem *decorations =
     new QTreeWidgetItem( mTreeWidget, QStringList(
                            i18nc( "@title:group", "Calendar Decorations" ) ) );
+#ifndef KORG_NOPRINTER
   QTreeWidgetItem *printPlugins =
     new QTreeWidgetItem( mTreeWidget, QStringList(
                            i18nc( "@title:group", "Print Plugins" ) ) );
+#endif
   QTreeWidgetItem *others =
     new QTreeWidgetItem( mTreeWidget, QStringList(
                            i18nc( "@title:group", "Other Plugins" ) ) );
 
   KService::List::ConstIterator it;
-  for ( it = plugins.begin(); it != plugins.end(); ++it ) {
+  for ( it = plugins.constBegin(); it != plugins.constEnd(); ++it ) {
     QTreeWidgetItem *item;
     if ( (*it)->hasServiceType( KOrg::CalendarDecoration::Decoration::serviceType() ) ) {
       item = new PluginItem( decorations, *it );
+#ifndef KORG_NOPRINTER
     } else if ( (*it)->hasServiceType( KOrg::PrintPlugin::serviceType() ) ){
       item = new PluginItem( printPlugins, *it );
+#endif
     } else {
       item = new PluginItem( others, *it );
     }
@@ -1319,7 +1387,9 @@ void KOPrefsDialogPlugins::usrReadConfig()
   }
 
   decorations->setExpanded( true );
+#ifndef KORG_NOPRINTER
   printPlugins->setExpanded( true );
+#endif
   others->setExpanded( true );
 
   mDecorationsAtMonthViewTop = KOPrefs::instance()->decorationsAtMonthViewTop().toSet();
@@ -1385,13 +1455,13 @@ void KOPrefsDialogPlugins::positioningChanged()
 
   QString decoration = item->service()->desktopEntryName();
 
-  if ( mPositionMonthTop->checkState() == Qt::Checked ) {
+  /*if ( mPositionMonthTop->checkState() == Qt::Checked ) {
     if ( !mDecorationsAtMonthViewTop.contains( decoration ) ) {
       mDecorationsAtMonthViewTop.insert( decoration );
     }
   } else {
     mDecorationsAtMonthViewTop.remove( decoration );
-  }
+  }*/
 
   if ( mPositionAgendaTop->isChecked() ) {
     if ( !mDecorationsAtAgendaViewTop.contains( decoration ) ) {
@@ -1415,7 +1485,7 @@ void KOPrefsDialogPlugins::positioningChanged()
 void KOPrefsDialogPlugins::selectionChanged()
 {
   mPositioningGroupBox->hide();
-  mPositionMonthTop->setChecked( false );
+  //mPositionMonthTop->setChecked( false );
   mPositionAgendaTop->setChecked( false );
   mPositionAgendaBottom->setChecked( false );
 
@@ -1450,10 +1520,10 @@ void KOPrefsDialogPlugins::selectionChanged()
   bool hasPosition = false;
   if ( item->service()->hasServiceType( KOrg::CalendarDecoration::Decoration::serviceType() ) ) {
     QString decoration = item->service()->desktopEntryName();
-    if ( mDecorationsAtMonthViewTop.contains( decoration ) ) {
+    /*if ( mDecorationsAtMonthViewTop.contains( decoration ) ) {
       mPositionMonthTop->setChecked( true );
       hasPosition = true;
-    }
+    }*/
     if ( mDecorationsAtAgendaViewTop.contains( decoration ) ) {
       mPositionAgendaTop->setChecked( true );
       hasPosition = true;
