@@ -32,22 +32,28 @@
 */
 
 #include "incidence.h"
+#include "resourcekolab.h"
 
-#include <qvaluelist.h>
+#include <QFile>
+#include <QList>
 
-#include <libkcal/journal.h>
+#include <kcal/journal.h>
 #include <korganizer/version.h>
 #include <kdebug.h>
+#include <kurl.h>
+#include <kio/netaccess.h>
 
 using namespace Kolab;
 
 
-Incidence::Incidence( const QString& tz, KCal::Incidence* incidence )
+Incidence::Incidence( KCal::ResourceKolab *res, const QString &subResource, quint32 sernum,
+                      const QString& tz, KCal::Incidence* incidence )
   : KolabBase( tz ), mFloatingStatus( Unset ), mHasAlarm( false ),
-    mRevision( 0 )
+    mRevision( 0 ),
+    mResource( res ),
+    mSubResource( subResource ),
+    mSernum( sernum )
 {
-  if ( incidence )
-    setFields( incidence );
 }
 
 Incidence::~Incidence()
@@ -84,19 +90,19 @@ KolabBase::Email Incidence::organizer() const
   return mOrganizer;
 }
 
-void Incidence::setStartDate( const QDateTime& startDate )
+void Incidence::setStartDate( const KDateTime& startDate )
 {
   mStartDate = startDate;
   if ( mFloatingStatus == AllDay )
-    kdDebug() << "ERROR: Time on start date but no time on the event\n";
+    kDebug() <<"ERROR: Time on start date but no time on the event";
   mFloatingStatus = HasTime;
 }
 
 void Incidence::setStartDate( const QDate& startDate )
 {
-  mStartDate = startDate;
+  mStartDate = KDateTime( startDate );
   if ( mFloatingStatus == HasTime )
-    kdDebug() << "ERROR: No time on start date but time on the event\n";
+    kDebug() <<"ERROR: No time on start date but time on the event";
   mFloatingStatus = AllDay;
 }
 
@@ -110,7 +116,7 @@ void Incidence::setStartDate( const QString& startDate )
     setStartDate( stringToDate( startDate ) );
 }
 
-QDateTime Incidence::startDate() const
+KDateTime Incidence::startDate() const
 {
   return mStartDate;
 }
@@ -136,24 +142,24 @@ void Incidence::addAttendee( const Attendee& attendee )
   mAttendees.append( attendee );
 }
 
-QValueList<Incidence::Attendee>& Incidence::attendees()
+QList<Incidence::Attendee>& Incidence::attendees()
 {
   return mAttendees;
 }
 
-const QValueList<Incidence::Attendee>& Incidence::attendees() const
+const QList<Incidence::Attendee>& Incidence::attendees() const
 {
   return mAttendees;
 }
 
-void Incidence::setSchedulingID( const QString& sid )
+void Incidence::setInternalUID( const QString& iuid )
 {
-  mSchedulingID = sid;
+  mInternalUID = iuid;
 }
 
-QString Incidence::schedulingID() const
+QString Incidence::internalUID() const
 {
-  return mSchedulingID;
+  return mInternalUID;
 }
 
 void Incidence::setRevision( int revision )
@@ -185,17 +191,21 @@ bool Incidence::loadAttendeeAttribute( QDomElement& element,
       else if ( tagName == "request-response" )
         // This sets reqResp to false, if the text is "false". Otherwise it
         // sets it to true. This means the default setting is true.
-        attendee.requestResponse = ( e.text().lower() != "false" );
+        attendee.requestResponse = ( e.text().toLower() != "false" );
       else if ( tagName == "invitation-sent" )
         // Like above, only this defaults to false
-        attendee.invitationSent = ( e.text().lower() != "true" );
+        attendee.invitationSent = ( e.text().toLower() != "true" );
       else if ( tagName == "role" )
         attendee.role = e.text();
+      else if ( tagName == "delegated-to" )
+        attendee.delegate = e.text();
+      else if ( tagName == "delegated-from" )
+        attendee.delegator = e.text();
       else
         // TODO: Unhandled tag - save for later storage
-        kdDebug() << "Warning: Unhandled tag " << e.tagName() << endl;
+        kDebug() <<"Warning: Unhandled tag" << e.tagName();
     } else
-      kdDebug() << "Node is not a comment or an element???" << endl;
+      kDebug() <<"Node is not a comment or an element???";
   }
 
   return true;
@@ -214,24 +224,85 @@ void Incidence::saveAttendeeAttribute( QDomElement& element,
   writeString( e, "invitation-sent",
                ( attendee.invitationSent ? "true" : "false" ) );
   writeString( e, "role", attendee.role );
+  writeString( e, "delegated-to", attendee.delegate );
+  writeString( e, "delegated-from", attendee.delegator );
 }
 
 void Incidence::saveAttendees( QDomElement& element ) const
 {
-  QValueList<Attendee>::ConstIterator it = mAttendees.begin();
-  for ( ; it != mAttendees.end(); ++it )
-    saveAttendeeAttribute( element, *it );
+  foreach ( const Attendee& attendee, mAttendees )
+    saveAttendeeAttribute( element, attendee );
 }
 
 void Incidence::saveAttachments( QDomElement& element ) const
 {
-  KCal::Attachment::List::ConstIterator it = mAttachments.begin();
-  for ( ; it != mAttachments.end(); ++it ) {
-    KCal::Attachment *a = (*it);
+  foreach ( KCal::Attachment *a, mAttachments ) {
     if ( a->isUri() ) {
       writeString( element, "link-attachment", a->uri() );
     } else if ( a->isBinary() ) {
-      // TODO
+      writeString( element, "inline-attachment", a->label() );
+    }
+  }
+}
+
+void Incidence::saveAlarms( QDomElement& element ) const
+{
+  if ( mAlarms.isEmpty() ) return;
+
+  QDomElement list = element.ownerDocument().createElement( "advanced-alarms" );
+  element.appendChild( list );
+  foreach ( KCal::Alarm *a, mAlarms ) {
+    QDomElement e = list.ownerDocument().createElement( "alarm" );
+    list.appendChild( e );
+
+    writeString( e, "enabled", a->enabled() ? "1" : "0" );
+    if ( a->hasStartOffset() ) {
+      writeString( e, "start-offset", QString::number( a->startOffset().asSeconds()/60 ) );
+    }
+    if ( a->hasEndOffset() ) {
+      writeString( e, "end-offset", QString::number( a->endOffset().asSeconds()/60 ) );
+    }
+    if ( a->repeatCount() ) {
+      writeString( e, "repeat-count", QString::number( a->repeatCount() ) );
+      writeString( e, "repeat-interval", QString::number( a->snoozeTime().asSeconds() ) );
+    }
+
+    switch ( a->type() ) {
+    case KCal::Alarm::Invalid:
+      break;
+    case KCal::Alarm::Display:
+      e.setAttribute( "type", "display" );
+      writeString( e, "text", a->text() );
+      break;
+    case KCal::Alarm::Procedure:
+      e.setAttribute( "type", "procedure" );
+      writeString( e, "program", a->programFile() );
+      writeString( e, "arguments", a->programArguments() );
+      break;
+    case KCal::Alarm::Email:
+    {
+      e.setAttribute( "type", "email" );
+      QDomElement addresses = e.ownerDocument().createElement( "addresses" );
+      e.appendChild( addresses );
+      foreach ( KCal::Person person, a->mailAddresses() ) {
+        writeString( addresses, "address", person.fullName() );
+      }
+      writeString( e, "subject", a->mailSubject() );
+      writeString( e, "mail-text", a->mailText() );
+      QDomElement attachments = e.ownerDocument().createElement( "attachments" );
+      e.appendChild( attachments );
+      foreach ( QString attachment, a->mailAttachments() ) {
+        writeString( attachments, "attachment", attachment );
+      }
+      break;
+    }
+    case KCal::Alarm::Audio:
+      e.setAttribute( "type", "audio" );
+      writeString( e, "file", a->audioFile() );
+      break;
+    default:
+      kWarning() << "Unhandled alarm type:" << a->type();
+      break;
     }
   }
 }
@@ -244,8 +315,8 @@ void Incidence::saveRecurrence( QDomElement& element ) const
   if ( !mRecurrence.type.isEmpty() )
     e.setAttribute( "type", mRecurrence.type );
   writeString( e, "interval", QString::number( mRecurrence.interval ) );
-  for( QStringList::ConstIterator it = mRecurrence.days.begin(); it != mRecurrence.days.end(); ++it ) {
-    writeString( e, "day", *it );
+  foreach ( const QString& recurrence, mRecurrence.days ) {
+    writeString( e, "day", recurrence );
   }
   if ( !mRecurrence.dayNumber.isEmpty() )
     writeString( e, "daynumber", mRecurrence.dayNumber );
@@ -258,9 +329,8 @@ void Incidence::saveRecurrence( QDomElement& element ) const
     QDomText t = element.ownerDocument().createTextNode( mRecurrence.range );
     range.appendChild( t );
   }
-  for( QValueList<QDate>::ConstIterator it = mRecurrence.exclusions.begin();
-       it != mRecurrence.exclusions.end(); ++it ) {
-    writeString( e, "exclusion", dateToString( *it ) );
+  foreach ( const QDate& date, mRecurrence.exclusions ) {
+    writeString( e, "exclusion", dateToString( date ) );
   }
 }
 
@@ -291,7 +361,119 @@ void Incidence::loadRecurrence( const QDomElement& element )
         mRecurrence.exclusions.append( stringToDate( e.text() ) );
       } else
         // TODO: Unhandled tag - save for later storage
-        kdDebug() << "Warning: Unhandled tag " << e.tagName() << endl;
+        kDebug() <<"Warning: Unhandled tag" << e.tagName();
+    }
+  }
+}
+
+static void loadAddressesHelper( const QDomElement& element, KCal::Alarm* a )
+{
+  for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
+    if ( n.isComment() )
+      continue;
+    if ( n.isElement() ) {
+      QDomElement e = n.toElement();
+      QString tagName = e.tagName();
+
+      if ( tagName == "address" ) {
+        a->addMailAddress( KCal::Person( e.text() ) );
+      } else {
+        kWarning() << "Unhandled tag" << tagName;
+      }
+    }
+  }
+}
+
+static void loadAttachmentsHelper( const QDomElement& element, KCal::Alarm* a )
+{
+  for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
+    if ( n.isComment() )
+      continue;
+    if ( n.isElement() ) {
+      QDomElement e = n.toElement();
+      QString tagName = e.tagName();
+
+      if ( tagName == "attachment" ) {
+        a->addMailAttachment( e.text() );
+      } else {
+        kWarning() << "Unhandled tag" << tagName;
+      }
+    }
+  }
+}
+
+static void loadAlarmHelper( const QDomElement& element, KCal::Alarm* a )
+{
+  for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
+    if ( n.isComment() )
+      continue;
+    if ( n.isElement() ) {
+      QDomElement e = n.toElement();
+      QString tagName = e.tagName();
+
+      if ( tagName == "start-offset" ) {
+        a->setStartOffset( e.text().toInt()*60 );
+      } else if ( tagName == "end-offset" ) {
+        a->setEndOffset( e.text().toInt()*60 );
+      } else if ( tagName == "repeat-count" ) {
+        a->setRepeatCount( e.text().toInt() );
+      } else if ( tagName == "repeat-interval" ) {
+        a->setSnoozeTime( e.text().toInt() );
+      } else if ( tagName == "text" ) {
+        a->setText( e.text() );
+      } else if ( tagName == "program" ) {
+        a->setProgramFile( e.text() );
+      } else if ( tagName == "arguments" ) {
+        a->setProgramArguments( e.text() );
+      } else if ( tagName == "addresses" ) {
+        loadAddressesHelper( e, a );
+      } else if ( tagName == "subject" ) {
+        a->setMailSubject( e.text() );
+      } else if ( tagName == "mail-text" ) {
+        a->setMailText( e.text() );
+      } else if ( tagName == "attachments" ) {
+        loadAttachmentsHelper( e, a );
+      } else if ( tagName == "file" ) {
+        a->setAudioFile( e.text() );
+      } else if ( tagName == "enabled" ) {
+        a->setEnabled( e.text().toInt() != 0 );
+      } else {
+        kWarning() << "Unhandled tag" << tagName;
+      }
+    }
+  }
+}
+
+void Incidence::loadAlarms( const QDomElement& element )
+{
+  for ( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
+    if ( n.isComment() )
+      continue;
+    if ( n.isElement() ) {
+      QDomElement e = n.toElement();
+      QString tagName = e.tagName();
+
+      if ( tagName == "alarm" ) {
+        KCal::Alarm *a = new KCal::Alarm( 0 );
+        a->setEnabled( true ); // default to enabled, unless some XML attribute says otherwise.
+        QString type = e.attribute( "type" );
+        if ( type == "display" ) {
+          a->setType( KCal::Alarm::Display );
+        } else if ( type == "procedure" ) {
+          a->setType( KCal::Alarm::Procedure );
+        } else if ( type == "email" ) {
+          a->setType( KCal::Alarm::Email );
+        } else if ( type == "audio" ) {
+          a->setType( KCal::Alarm::Audio );
+        } else {
+          kWarning() << "Unhandled alarm type:" << type;
+        }
+
+        loadAlarmHelper( e, a );
+        mAlarms << a;
+      } else {
+        kWarning() << "Unhandled tag" << tagName;
+      }
     }
   }
 }
@@ -322,15 +504,15 @@ bool Incidence::loadAttribute( QDomElement& element )
       return true;
     } else
       return false;
-  } else if ( tagName == "inline-attachment" ) {
-    // TODO
   } else if ( tagName == "link-attachment" ) {
     mAttachments.push_back( new KCal::Attachment( element.text() ) );
   } else if ( tagName == "alarm" )
     // Alarms should be minutes before. Libkcal uses event time + alarm time
     setAlarm( - element.text().toInt() );
-  else if ( tagName == "scheduling-id" )
-    setSchedulingID( element.text() );
+  else if ( tagName == "advanced-alarms" )
+    loadAlarms( element );
+  else if ( tagName == "x-kde-internaluid" )
+    setInternalUID( element.text() );
   else if ( tagName == "revision" ) {
     bool ok;
     int revision = element.text().toInt( &ok );
@@ -342,9 +524,9 @@ bool Incidence::loadAttribute( QDomElement& element )
     bool ok = KolabBase::loadAttribute( element );
     if ( !ok ) {
         // Unhandled tag - save for later storage
-        kdDebug() << "Saving unhandled tag " << element.tagName() << endl;
+        kDebug() <<"Saving unhandled tag" << element.tagName();
         Custom c;
-        c.key = QCString( "X-KDE-KolabUnhandled-" ) + element.tagName().latin1();
+        c.key = QByteArray( "X-KDE-KolabUnhandled-" ) + element.tagName().toLatin1();
         c.value = element.text();
         mCustomList.append( c );
     }
@@ -374,7 +556,8 @@ bool Incidence::saveAttributes( QDomElement& element ) const
     int alarmTime = qRound( -alarm() );
     writeString( element, "alarm", QString::number( alarmTime ) );
   }
-  writeString( element, "scheduling-id", schedulingID() );
+  saveAlarms( element );
+  writeString( element, "x-kde-internaluid", internalUID() );
   writeString( element, "revision", QString::number( revision() ) );
   saveCustomAttributes( element );
   return true;
@@ -382,19 +565,18 @@ bool Incidence::saveAttributes( QDomElement& element ) const
 
 void Incidence::saveCustomAttributes( QDomElement& element ) const
 {
-  QValueList<Custom>::ConstIterator it = mCustomList.begin();
-  for ( ; it != mCustomList.end(); ++it ) {
-    QString key = (*it).key;
+  foreach ( const Custom& custom, mCustomList ) {
+    QString key( custom.key );
     Q_ASSERT( !key.isEmpty() );
     if ( key.startsWith( "X-KDE-KolabUnhandled-" ) ) {
       key = key.mid( strlen( "X-KDE-KolabUnhandled-" ) );
-      writeString( element, key, (*it).value );
+      writeString( element, key, custom.value );
     } else {
       // Let's use attributes so that other tag-preserving-code doesn't need sub-elements
       QDomElement e = element.ownerDocument().createElement( "x-custom" );
       element.appendChild( e );
       e.setAttribute( "key", key );
-      e.setAttribute( "value", (*it).value );
+      e.setAttribute( "value", custom.value );
     }
   }
 }
@@ -402,7 +584,7 @@ void Incidence::saveCustomAttributes( QDomElement& element ) const
 void Incidence::loadCustomAttributes( QDomElement& element )
 {
   Custom custom;
-  custom.key = element.attribute( "key" ).latin1();
+  custom.key = element.attribute( "key" ).toLatin1();
   custom.value = element.attribute( "value" );
   mCustomList.append( custom );
 }
@@ -415,6 +597,12 @@ static KCal::Attendee::PartStat attendeeStringToStatus( const QString& s )
     return KCal::Attendee::Tentative;
   if ( s == "declined" )
     return KCal::Attendee::Declined;
+  if ( s == "delegated" )
+    return KCal::Attendee::Delegated;
+  if ( s == "needs-action" ) {
+    // kolab 2.2.0 sets needs-action (invalid according the standard),
+    return KCal::Attendee::None;
+  }
 
   // Default:
   return KCal::Attendee::Accepted;
@@ -432,10 +620,13 @@ static QString attendeeStatusToString( KCal::Attendee::PartStat status )
   case KCal::Attendee::Tentative:
     return "tentative";
   case KCal::Attendee::Delegated:
+    return "delegated";
   case KCal::Attendee::Completed:
   case KCal::Attendee::InProcess:
     // These don't have any meaning in the Kolab format, so just use:
     return "accepted";
+  case KCal::Attendee::None:
+    return "unknown";
   }
 
   // Default for the case that there are more added later:
@@ -506,7 +697,7 @@ void Incidence::setRecurrence( KCal::Recurrence* recur )
   case KCal::Recurrence::rMonthlyPos: {
     mRecurrence.cycle = "monthly";
     mRecurrence.type = "weekday";
-    QValueList<KCal::RecurrenceRule::WDayPos> monthPositions = recur->monthPositions();
+    QList<KCal::RecurrenceRule::WDayPos> monthPositions = recur->monthPositions();
     if ( !monthPositions.isEmpty() ) {
       KCal::RecurrenceRule::WDayPos monthPos = monthPositions.first();
       // TODO: Handle multiple days in the same week
@@ -519,7 +710,7 @@ void Incidence::setRecurrence( KCal::Recurrence* recur )
   case KCal::Recurrence::rMonthlyDay: {
     mRecurrence.cycle = "monthly";
     mRecurrence.type = "daynumber";
-    QValueList<int> monthDays = recur->monthDays();
+    QList<int> monthDays = recur->monthDays();
     // ####### Kolab XML limitation: only the first month day is used
     if ( !monthDays.isEmpty() )
       mRecurrence.dayNumber = QString::number( monthDays.first() );
@@ -529,10 +720,10 @@ void Incidence::setRecurrence( KCal::Recurrence* recur )
   {
     mRecurrence.cycle = "yearly";
     mRecurrence.type = "monthday";
-    QValueList<int> rmd = recur->yearDates();
+    QList<int> rmd = recur->yearDates();
     int day = !rmd.isEmpty() ? rmd.first() : recur->startDate().day();
     mRecurrence.dayNumber = QString::number( day );
-    QValueList<int> months = recur->yearMonths();
+    QList<int> months = recur->yearMonths();
     if ( !months.isEmpty() )
       mRecurrence.month = s_monthName[ months.first() - 1 ]; // #### Kolab XML limitation: only one month specified
     break;
@@ -545,10 +736,10 @@ void Incidence::setRecurrence( KCal::Recurrence* recur )
   case KCal::Recurrence::rYearlyPos: // (weekday X of week N of month Y)
     mRecurrence.cycle = "yearly";
     mRecurrence.type = "weekday";
-    QValueList<int> months = recur->yearMonths();
+    QList<int> months = recur->yearMonths();
     if ( !months.isEmpty() )
       mRecurrence.month = s_monthName[ months.first() - 1 ]; // #### Kolab XML limitation: only one month specified
-    QValueList<KCal::RecurrenceRule::WDayPos> monthPositions = recur->yearPositions();
+    QList<KCal::RecurrenceRule::WDayPos> monthPositions = recur->yearPositions();
     if ( !monthPositions.isEmpty() ) {
       KCal::RecurrenceRule::WDayPos monthPos = monthPositions.first();
       // TODO: Handle multiple days in the same week
@@ -576,8 +767,8 @@ void Incidence::setFields( const KCal::Incidence* incidence )
 {
   KolabBase::setFields( incidence );
 
-  if ( incidence->doesFloat() ) {
-    // This is a floating event. Don't timezone move this one
+  if ( incidence->allDay() ) {
+    // This is a all-day event. Don't timezone move this one
     mFloatingStatus = AllDay;
     setStartDate( incidence->dtStart().date() );
   } else {
@@ -606,9 +797,7 @@ void Incidence::setFields( const KCal::Incidence* incidence )
 
   // Attendees:
   KCal::Attendee::List attendees = incidence->attendees();
-  KCal::Attendee::List::ConstIterator it;
-  for ( it = attendees.begin(); it != attendees.end(); ++it ) {
-    KCal::Attendee* kcalAttendee = *it;
+  foreach ( KCal::Attendee *kcalAttendee, attendees ) {
     Attendee attendee;
 
     attendee.displayName = kcalAttendee->name();
@@ -619,6 +808,8 @@ void Incidence::setFields( const KCal::Incidence* incidence )
     // attendee.invitationSent = kcalAttendee->mFlag;
     // DF: Hmm? mFlag is set to true and never used at all.... Did you mean another field?
     attendee.role = attendeeRoleToString( kcalAttendee->role() );
+    attendee.delegate = kcalAttendee->delegate();
+    attendee.delegator = kcalAttendee->delegator();
 
     addAttendee( attendee );
   }
@@ -627,31 +818,42 @@ void Incidence::setFields( const KCal::Incidence* incidence )
 
   // Attachments
   KCal::Attachment::List attachments = incidence->attachments();
-  KCal::Attachment::List::ConstIterator it2;
-  for ( it2 = attachments.begin(); it2 != attachments.end(); ++it2 ) {
-    KCal::Attachment *a = *it2;
+  foreach ( KCal::Attachment* a, attachments ) {
     mAttachments.push_back( a );
   }
 
-  if ( incidence->doesRecur() ) {
+  mAlarms.clear();
+
+  // Alarms
+  KCal::Alarm::List alarms = incidence->alarms();
+  foreach ( KCal::Alarm* a, alarms ) {
+    mAlarms.push_back( a );
+  }
+
+  if ( incidence->recurs() ) {
     setRecurrence( incidence->recurrence() );
     mRecurrence.exclusions = incidence->recurrence()->exDates();
   }
 
   // Handle the scheduling ID
-  if ( incidence->schedulingID() == incidence->uid() )
+  if ( incidence->schedulingID() == incidence->uid() ) {
     // There is no scheduling ID
-    setSchedulingID( QString::null );
-  else
-    setSchedulingID( incidence->schedulingID() );
+    setInternalUID( QString::null );	//krazy:exclude=nullstrassign for old broken gcc
+  } else {
+    // We've internally been using a different uid, so save that as the
+    // temporary (internal) uid and restore the original uid, the one that
+    // is used in the folder and the outside world
+    setUid( incidence->schedulingID() );
+    setInternalUID( incidence->uid() );
+  }
 
   // Unhandled tags and other custom properties (see libkcal/customproperties.h)
-  const QMap<QCString, QString> map = incidence->customProperties();
-  QMap<QCString, QString>::ConstIterator cit = map.begin();
+  const QMap<QByteArray, QString> map = incidence->customProperties();
+  QMap<QByteArray, QString>::ConstIterator cit = map.begin();
   for ( ; cit != map.end() ; ++cit ) {
     Custom c;
     c.key = cit.key();
-    c.value = cit.data();
+    c.value = cit.value();
     mCustomList.append( c );
   }
 }
@@ -660,9 +862,9 @@ static QBitArray daysListToBitArray( const QStringList& days )
 {
   QBitArray arr( 7 );
   arr.fill( false );
-  for( QStringList::ConstIterator it = days.begin(); it != days.end(); ++it ) {
+  foreach ( const QString& day, days ) {
     for ( uint i = 0; i < 7 ; ++i )
-      if ( *it == s_weekDayName[i] )
+      if ( day == s_weekDayName[i] )
         arr.setBit( i, true );
   }
   return arr;
@@ -674,44 +876,50 @@ void Incidence::saveTo( KCal::Incidence* incidence )
   KolabBase::saveTo( incidence );
 
   if ( mFloatingStatus == AllDay ) {
-    // This is a floating event. Don't timezone move this one
+    // This is an all-day event. Don't timezone move this one
     incidence->setDtStart( startDate() );
-    incidence->setFloats( true );
+    incidence->setAllDay( true );
   } else {
     incidence->setDtStart( utcToLocal( startDate() ) );
-    incidence->setFloats( false );
+    incidence->setAllDay( false );
   }
 
   incidence->setSummary( summary() );
   incidence->setLocation( location() );
 
-  if ( mHasAlarm ) {
+  if ( mHasAlarm && mAlarms.isEmpty() ) {
     KCal::Alarm* alarm = incidence->newAlarm();
     alarm->setStartOffset( qRound( mAlarm * 60.0 ) );
     alarm->setEnabled( true );
+    alarm->setType( KCal::Alarm::Display );
+  } else if ( !mAlarms.isEmpty() ) {
+    foreach ( KCal::Alarm* a, mAlarms ) {
+      a->setParent( incidence );
+      incidence->addAlarm( a );
+    }
   }
 
   if ( organizer().displayName.isEmpty() )
     incidence->setOrganizer( organizer().smtpAddress );
   else
-    incidence->setOrganizer( organizer().displayName + "<"
-                             + organizer().smtpAddress + ">" );
+    incidence->setOrganizer( organizer().displayName + '<'
+                             + organizer().smtpAddress + '>' );
 
   incidence->clearAttendees();
-  QValueList<Attendee>::ConstIterator it;
-  for ( it = mAttendees.begin(); it != mAttendees.end(); ++it ) {
-    KCal::Attendee::PartStat status = attendeeStringToStatus( (*it).status );
-    KCal::Attendee::Role role = attendeeStringToRole( (*it).role );
-    incidence->addAttendee( new KCal::Attendee( (*it).displayName,
-                                                (*it).smtpAddress,
-                                                (*it).requestResponse,
-                                                status, role ) );
+  foreach ( const Attendee& attendee, mAttendees ) {
+    KCal::Attendee::PartStat status = attendeeStringToStatus( attendee.status );
+    KCal::Attendee::Role role = attendeeStringToRole( attendee.role );
+    KCal::Attendee *a = new KCal::Attendee( attendee.displayName,
+                                            attendee.smtpAddress,
+                                            attendee.requestResponse,
+                                            status, role );
+    a->setDelegate( attendee.delegate );
+    a->setDelegator( attendee.delegator );
+    incidence->addAttendee( a );
   }
 
   incidence->clearAttachments();
-  KCal::Attachment::List::ConstIterator it2;
-  for ( it2 = mAttachments.begin(); it2 != mAttachments.end(); ++it2 ) {
-    KCal::Attachment *a = (*it2);
+  foreach ( KCal::Attachment *a, mAttachments ) {
     // TODO should we copy?
     incidence->addAttachment( a );
   }
@@ -734,7 +942,7 @@ void Incidence::saveTo( KCal::Incidence* incidence )
         recur->addMonthlyPos( mRecurrence.dayNumber.toInt(), daysListToBitArray( mRecurrence.days ) );
       } else if ( mRecurrence.type == "daynumber" ) {
         recur->addMonthlyDate( mRecurrence.dayNumber.toInt() );
-      } else kdWarning() << "Unhandled monthly recurrence type " << mRecurrence.type << endl;
+      } else kWarning() <<"Unhandled monthly recurrence type" << mRecurrence.type;
     } else if ( mRecurrence.cycle == "yearly" ) {
       recur->setYearly( mRecurrence.interval );
       if ( mRecurrence.type == "monthday" ) {
@@ -749,8 +957,8 @@ void Incidence::saveTo( KCal::Incidence* incidence )
           if ( s_monthName[ i ] == mRecurrence.month )
             recur->addYearlyMonth( i+1 );
         recur->addYearlyPos( mRecurrence.dayNumber.toInt(), daysListToBitArray( mRecurrence.days ) );
-      } else kdWarning() << "Unhandled yearly recurrence type " << mRecurrence.type << endl;
-    } else kdWarning() << "Unhandled recurrence cycle " << mRecurrence.cycle << endl;
+      } else kWarning() <<"Unhandled yearly recurrence type" << mRecurrence.type;
+    } else kWarning() <<"Unhandled recurrence cycle" << mRecurrence.cycle;
 
     if ( mRecurrence.rangeType == "number" ) {
       recur->setDuration( mRecurrence.range.toInt() );
@@ -759,21 +967,54 @@ void Incidence::saveTo( KCal::Incidence* incidence )
     } // "none" is default since tje set*ly methods set infinite recurrence
 
     incidence->recurrence()->setExDates( mRecurrence.exclusions );
+
+  }
+  /* If we've stored a uid to be used internally instead of the real one
+   * (to deal with duplicates of events in different folders) before, then
+   * restore it, so it does not change. Keep the original uid around for
+   * scheduling purposes. */
+  if ( !internalUID().isEmpty() ) {
+    incidence->setUid( internalUID() );
+    incidence->setSchedulingID( uid() );
   }
 
-  incidence->setSchedulingID( schedulingID() );
-
-  for( QValueList<Custom>::ConstIterator it = mCustomList.begin(); it != mCustomList.end(); ++it ) {
-    incidence->setNonKDECustomProperty( (*it).key, (*it).value );
+  foreach ( const Custom& custom, mCustomList ) {
+    incidence->setNonKDECustomProperty( custom.key, custom.value );
   }
 
 }
 
+void Incidence::loadAttachments()
+{
+  QStringList attachments;
+  if ( mResource->kmailListAttachments( attachments, mSubResource, mSernum ) ) {
+    foreach ( const QString& attachment, attachments ) {
+      QByteArray data;
+      KUrl url;
+      if ( mResource->kmailGetAttachment( url, mSubResource, mSernum, attachment ) && !url.isEmpty() ) {
+        QFile f( url.toLocalFile() );
+        if ( f.open( QFile::ReadOnly ) ) {
+          data = f.readAll();
+          QString mimeType;
+          if ( !mResource->kmailAttachmentMimetype( mimeType, mSubResource, mSernum, attachment ) )
+            mimeType = "application/octet-stream";
+          KCal::Attachment *a = new KCal::Attachment( data.toBase64().constData(), mimeType );
+          a->setLabel( attachment );
+          mAttachments.append( a );
+          f.close();
+        }
+        f.remove();
+      }
+    }
+  }
+}
+
 QString Incidence::productID() const
 {
-  return QString( "KOrganizer " ) + korgVersion + ", Kolab resource";
+  return QString( "KOrganizer %1, Kolab resource" ).arg( korgVersion );
 }
 
 // Unhandled KCal::Incidence fields:
 // revision, status (unused), priority (done in tasks), attendee.uid,
 // mComments, mReadOnly
+
